<?php
/**
 * Plugin Name: OBERON - Hotel reservation
 * Plugin URI: https://exalogic.sk/na-stiahnutie-oberon/oberon-hotel-reservation-wordpress/
 * Text Domain: exalogic-oberon-hotel-reservation
 * Domain Path: /languages
 * Description: Connect reservation form from OBERON Web API to WordPress page. This widget connects to API on the background to protect the company system from the outside breach in to internal company infrastructure.
 * Version: 1.4.4
 * Author: Mário Moravčík, Exalogic, s.r.o.
 * Author URI: http://exalogic.sk
 * License: GPL2
 */

// Definovať potrebné konštanty.
if ( ! defined( 'OBERON_TEXT_DOMAIN' ) ) {
	define( 'OBERON_TEXT_DOMAIN', 'exalogic-oberon-texts' );
}
define( 'OBERONRESERVATION_VERSION', '1400' );
define( 'OBERONRESERVATION_TEXT_DOMAIN', 'exalogic-oberon-hotel-reservation' );
define( 'OBERONRESERVATION_URL', plugin_dir_url( __FILE__ ) );
define( 'OBERONRESERVATION_PATH', plugin_dir_path( __FILE__ ) );
define( 'OBERONRESERVATION_OPTION_NAME', 'exalogic_oberonreservation_settings' );
define( 'OBERONRESERVATION_OPTION_VERSION', 'exalogic_oberonreservation_version' );
define( 'OBERONRESERVATION_SHORTCODE', 'oberon_hotel_reservation' );


if ( ! function_exists( 'add_action' ) ) {
	echo 'Hi there! This is a plugin, when called directly it\'s not working.';
	exit;
}


// Register widget.
require_once dirname( __FILE__ ) . '/exalogic_oberon_hotel_reservation_plugin_admin.php';
require_once dirname( __FILE__ ) . '/exalogic_oberon_hotel_reservation_widget.php';
require_once dirname( __FILE__ ) . '/exalogic_oberon_hotel_reservation_rest_controller.php';


if ( is_admin() ) {
	// init oberon admin.
	$oberon_hotel_reservation_admin = new OBERON_HotelReservation_Admin();
}


// Register handler for registering of widgets.
add_action( 'widgets_init', 'oberon_hotelreservation_register_widgets' );
/**
 * Widget register.
 */
function oberon_hotelreservation_register_widgets() {

	register_widget( 'OBERON_HotelReservation' ); // Must be the same as BaseID in widget.

}


// Add [oberon_hotel_reservation] shortcode for pages with no widget areas.
add_shortcode( OBERONRESERVATION_SHORTCODE, 'oberon_hotelreservation_widget_shortcode' );


/**
 * Widget [oberon_hotel_reservation] shortcode handler.
 *
 * Accepts parameters:
 * lang="" ISO 2 character language code for languages available in lang directory.
 * 
 * @param array  $atts    Shortcode attributes. Default empty.
 * @param string $content Shortcode content. Default null.
 * @param string $tag     Shortcode tag (name). Default empty.
 * @return string Shortcode output.
 */
function oberon_hotelreservation_widget_shortcode( $atts = array(), $content = null, $tag = '' ) {

	global $wp_widget_factory;

	//oberon-hotel-reservation-form

	$widget_name = 'OBERON_HotelReservation';

	// extract( shortcode_atts( array(
	// 	'widget_name' => false,
	// ), $atts ) );

	// $widget_name = wp_specialchars( $widget_name );

	if ( ! is_a( $wp_widget_factory->widgets[ $widget_name ], 'WP_Widget' ) ) {
		 $wp_class = 'WP_Widget_' . ucwords( strtolower( $class ) );
		if ( ! is_a( $wp_widget_factory->widgets[ $wp_class ], 'WP_Widget' ) ) {
			return '<p>' . sprintf( __( "%s: Widget class not found. Make sure this widget exists and the class name is correct", OBERONRESERVATION_TEXT_DOMAIN ),'<strong>' . $class . '</strong>' ) . '</p>';
		} else {
			$class = $wp_class;
		}
	}

	ob_start();
	the_widget( $widget_name, $atts, array(
		'widget_id' => 'arbitrary-instance-' . $id,
		'before_widget' => '',
		'after_widget' => '',
		'before_title' => '',
		'after_title' => '',
		'content' => $content,
		'tag' => $tag,
	));
	$output = ob_get_contents();
	ob_end_clean();
	return $output;

}
