<?php
/**
 * OBERON_HotelReservation - Widget class.
 * 
 * This file is part of the OBERON System and WordPress plugins made by Exalogic company and is released under the same license.
 * For more information please see oberon_hotel_reservation_plugin.php.
 * 
 * Copyright (c) 2007-2020 Mário Moravcik. All rights reserved.
 * 
 * Version: 1.4.4
 */

class OBERON_HotelReservation_RouteController extends WP_REST_Controller {

	/**
	 * Here initialize our namespace and resource name.
	 */
	public function __construct() {
		/*
		 Watch for changing version this need to be changed also in script file 
		 'exalogic-oberon-hotel-reservation.js' and 'exalogic-oberon-reservation-plugin-admin.js' - '_self.info.version: "1.1"'
		*/
		$this->version = '1.1';
		$this->namespace     = '/oberonweb/v' . $this->version;
		$this->resource_name = 'hotel';
		$this->settings = null;

		$this->load_settings();
	}

	/**
	 * Load settings.
	 */
	private function load_settings() {

		$settings = get_option( OBERONRESERVATION_OPTION_NAME, array() );

		$this->settings = (object) array(
			'oberon_web_url' => (object) array(
				'control_id' => 'oberon-web-url',
				'value_type' => 'text',
				'value' => '',
			),
			'price_category_guest' => (object) array(
				'control_id' => 'price-category-guest',
				'value_type' => 'text',
				'value' => '',
			),
			'allow_roomtype_change' => (object) array(
				'control_id' => 'allow-roomtype-change',
				'value_type' => 'numeric[0/1]',
				'value' => 1,
			),
			'calculate_accommodation' => (object) array(
				'control_id' => 'calculate-accommodation',
				'value_type' => 'numeric[0/1]',
				'value' => 1,
			),
			'calculate_localtax' => (object) array(
				'control_id' => 'calculate-localtax',
				'value_type' => 'numeric[0/1]',
				'value' => 0,
			),
			'calculate_services' => (object) array(
				'control_id' => 'calculate-services',
				'value_type' => 'numeric[0/1]',
				'value' => 0,
			),
		);

		$this->settings->oberon_web_url->value              = untrailingslashit( $settings[ OBERON_HotelReservation_Admin::CONST_SERVICE_URL_NAME ] );
		$this->settings->calculate_accommodation->value     = $settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_ACCOMMODATION_NAME ];
		$this->settings->calculate_localtax->value          = $settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_LOCALTAX_NAME ];
		$this->settings->calculate_services->value          = $settings[ OBERON_HotelReservation_Admin::CONST_CALCULATE_SERVICES_NAME ];
		$this->settings->allow_roomtype_change->value       = $settings[ OBERON_HotelReservation_Admin::CONST_ROOM_TYPE_CHANGE_NAME ];
		$this->settings->price_category_guest->value        = untrailingslashit( $settings[ OBERON_HotelReservation_Admin::CONST_PRICE_CATEGORY_GUEST_NAME ] );

	}

	/**
	 * Get base oberon url.
	 */
	private function get_oberon_web_url() {
		return $this->settings->oberon_web_url->value;
	}

	/**
	 * Get reservation form calculation flag for accommodation.
	 */
	private function get_reservation_calc_accommodation() {
		return $this->settings->calculate_accommodation->value;
	}

	/**
	 * Get reservation form calculation flag for local tax.
	 */
	private function get_reservation_calc_localtax() {
		return $this->settings->calculate_localtax->value;
	}

	/**
	 * Get reservation form calculation flag for basic services.
	 */
	private function get_reservation_calc_services() {
		return $this->settings->calculate_services->value;
	}

	/**
	 * Property value allow room type change flag.
	 */
	public function get_allow_roomtype_change() {
		return $this->settings->allow_roomtype_change->value;
	}

	/**
	 * Property value price category guest for calculation.
	 */
	public function get_price_category_guest() {
		return $this->settings->price_category_guest->value;
	}

	/**
	 * Get base route.
	 */
	private function get_base_route() {
		return  '/' . $this->resource_name;
	}

	/**
	 * Register the routes for the objects of the controller.
	 */
	public function register_routes() {

		// Only used in admin mode.
		register_rest_route( $this->namespace, $this->get_base_route() . '/getPluginSettings' , array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'admin_pluginSettings' ),
				'permission_callback' => array( $this, 'get_admin_permissions_check' ),
				'args'                => array(),
			),
		) );


		register_rest_route( $this->namespace, $this->get_base_route() . '/version' , array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_version' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( $this->namespace, $this->get_base_route() . '/ping' , array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'ping' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( $this->namespace, $this->get_base_route() . '/GetHotel_RoomTypes' , array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_hotel_roomtypes' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( $this->namespace, $this->get_base_route() . '/GetHotel_PriceCategoryGuests' , array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'get_hotel_pricecategoryguests' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( $this->namespace, $this->get_base_route() . '/GetHotel_Rooms' , array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_hotel_rooms' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( $this->namespace, $this->get_base_route() . '/GetHotel_ReservationAvailability' , array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'get_hotel_reservation_availability' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( $this->namespace, $this->get_base_route() . '/GetHotel_RoomReservationAvailability' , array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'get_hotel_room_reservation_availability' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( $this->namespace, $this->get_base_route() . '/SetHotel_Reservation' , array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'set_hotel_reservation' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( $this->namespace, $this->get_base_route() . '/SetHotel_RoomReservation' , array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'set_hotel_room_reservation' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => array(),
			),
		) );

	}


	/*
	--------------------------------------------------------------------.
	---  METHODS USED ONLY IN PLUGIN ADMINISTRATION. ---
	--------------------------------------------------------------------.
	*/
	/**
	 * Get data on this wordpress plugin settings.
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function admin_pluginSettings( $request ) {

		// Encode json back to request.
		$data = $this->get_exc_default();
		$data['result'] = true;
		$data['data'] = $this->settings;
		// Hide the IP.
		$data['data']->oberon_web_url->value = "Hidden in json";
		//$json = json_encode( $data ); //Not needed WP_REST_Response does this for you.
		//echo $json;

		return new WP_REST_Response( $data );

	}


	/**
	 * Test server availability.
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function ping( $request ) {

		$url = $this->get_oberon_web_url() . '/ping';
		$args = '';

		$response = wp_remote_get( $url, $args );

		$data = $this->get_exc_default();
		$body = wp_remote_retrieve_body( $response );

		if( ! $response || ! $body ) {
			$data[ 'result' ] = false;
			$data[ 'data' ] = 'Hotel web service not responding.';
		} else {
			$data[ 'result' ] = true;
			$data[ 'data' ] = $body;
		}	
		return new WP_REST_Response( $data, 200 );

	}


	/**
	 * Get version of oberon web service api.
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_version( $request ) {
		$user_data = '';
		$url = $this->get_oberon_web_url() . '/GetVersion';

		$myex = $this->get_data( $url );

		$headers = wp_remote_retrieve_headers( $response );
		$user_data = $headers['userData'];

		$response = new WP_REST_Response( $myex, 200 );
		$response->header( 'userData', $user_data );

		return $response;
	}


	/**
	 * Get a collection of room types.
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_hotel_roomtypes( $request ) {

		$this->load_settings();

		$url = $this->get_oberon_web_url() . '/GetHotel_RoomTypes';

		$myex = $this->get_data( $url );

		$headers = wp_remote_retrieve_headers( $response );
		$user_data = $headers['userData'];

		$response = new WP_REST_Response( $myex, 200 );
		$response->header( 'userData', $user_data );

		return $response;
	}

	/**
	 * Get a collection of guest price categories.
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_hotel_pricecategoryguests( $request ) {
		$this->load_settings();

		$myex = $this->get_exc_default();

		$url = $this->get_oberon_web_url() . '/GetHotel_PriceCategoryGuests';

		$body = $request->get_body();
		// server handles body if not as should be
		$args = array(
			'headers'   => array( 'Content-Type' => 'application/json; charset=utf-8' ),
			'body'      => $body,
			'method'    => 'POST',
		);

		$myex = $this->post_data( $url, $args );

		$headers = wp_remote_retrieve_headers( $response );
		$user_data = $headers['userData'];

		$response = new WP_REST_Response( $myex, 200 );
		$response->header( 'userData', $user_data );

		return $response;
	}

	/**
	 * Get a collection of items
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_hotel_rooms( $request ) {

		$url = $this->get_oberon_web_url() . '/GetHotel_Rooms';
		$args = '';

		$myex = $this->get_data( $url );

		$headers = wp_remote_retrieve_headers( $response );
		$user_data = $headers['userData'];

		$response = new WP_REST_Response( $myex, 200 );
		$response->header( 'userData', $user_data );

		return $response;
	}

	/**
	 * Get a collection of items
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_hotel_reservation_availability( $request ) {

		$myex = array(
			'result'        => false,
			'errNumber'     => 0,
			'description'   => '',
			'data'          => array(),
		);

		$url = $this->get_oberon_web_url() . '/GetHotel_ReservationAvailability';
		$body = $request->getBody();
		// server handles body if not as should be
		$args = array(
			'headers'   => array( 'Content-Type' => 'application/json; charset=utf-8' ),
			'body'      => $body,  //json_encode( $array_with_parameters ),
			'method'    => 'POST',
		);

		$myex = $this->post_data( $url, $args );

		$headers = wp_remote_retrieve_headers( $response );
		$user_data = $headers['userData'];

		$response = new WP_REST_Response( $myex, 200 );
		$response->header( 'userData', $user_data );

		return $response;

	}


	/**
	 * Get a collection of items
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_hotel_room_reservation_availability( $request ) {
		$myex = $this->get_exc_default();

		$url = $this->get_oberon_web_url() . '/GetHotel_RoomReservationAvailability';
		$body = $request->get_body();
		// Parse json body to change some settings.
		$json = json_decode( $body );
		// Plugin settings for price calculation.
		$json->{'HotelRoomReservationAvailabilityArg'}->{'PriceCalculate'} = ( 1 === $this->settings->calculate_accommodation->value ) ? true : false;
		// Setting for allowing the change of room type on availability check.
		$json->{'HotelRoomReservationAvailabilityArg'}->{'RoomTypeAllowChange'} = ( 1 === $this->settings->allow_roomtype_change->value ) ? true : false;
		// Common guest category name for price calculation control.
		$json->{'HotelRoomReservationAvailabilityArg'}->{'PriceCategoryGuestName'} = $this->settings->price_category_guest->value;
		// Encode json back to request.
		$body = json_encode( $json );
		$args = array(
			'headers'   => array( 'Content-Type' => 'application/json; charset=utf-8' ),
			'body'      => $body,
			'method'    => 'POST',
		);

		$myex = $this->post_data( $url, $args );

		$headers = wp_remote_retrieve_headers( $response );
		$user_data = $headers['userData'];

		$response = new WP_REST_Response( $myex, 200 );
		$response->header( 'userData', $user_data );

		return $response;

	}

	/**
	 * Get one item from the collection
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function set_hotel_reservation( $request ) {
		$myex = $this->get_exc_default();

		$url = $this->get_oberon_web_url() . '/SetHotel_Reservation';
		$body = $request->get_body();

		$args = array(
			'headers'   => array( 'Content-Type' => 'application/json; charset=utf-8' ),
			'body'      => $body,  //json_encode( $array_with_parameters ),
			'method'    => 'POST',
		);

		$myex = $this->post_data( $url, $args );

		$headers = wp_remote_retrieve_headers( $response );
		$user_data = $headers['userData'];

		$response = new WP_REST_Response( $myex, 200 );
		$response->header( 'userData', $user_data );

		return $response;
	}

	/**
	 * Get one item from the collection
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function set_hotel_room_reservation( $request ) {
		$myex = $this->get_exc_default();

		$url = $this->get_oberon_web_url() . '/SetHotel_RoomReservation';
		$body = $request->get_body();

		$args = array(
			'headers'   => array( 'Content-Type' => 'application/json; charset=utf-8' ),
			'body'      => $body,  //json_encode( $array_with_parameters ),
			'method'    => 'POST',
		);

		$myex = $this->post_data( $url, $args );

		$headers = wp_remote_retrieve_headers( $response );
		$user_data = $headers['userData'];

		$response = new WP_REST_Response( $myex, 200 );
		$response->header( 'userData', $user_data );

		return $response;
	}

	/* ----------  PERMISSIONS  ---------- */

	/**
	 * Check if a given request has access to get items
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|bool
	 */
	public function get_items_permissions_check( $request ) {
		//return true; <--use to make readable by all
		return true; //current_user_can( 'install_plugins' );
	}

	/**
	 * Check if a given request has access to get items
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 * @return WP_Error|bool
	 */
	public function get_admin_permissions_check( $request ) {
		//if ( is_admin() ) {
			return true; //current_user_can( 'install_plugins' );
		//} else {
		//	return false;
		//}			
	}


	/* ----------  DATA PREPARATION  ---------- */

	/**
	 * Prepare the item for the REST response
	 *
	 * @param mixed $item WordPress representation of the item.
	 * @param WP_REST_Request $request Request object.
	 * @return mixed
	 */
	public function prepare_item_for_response( $item, $request ) {
		return array();
	}

	/* ----------  PERMISSIONS  ---------- */

	/**
	 * Get the query params for collections
	 *
	 * @return array
	 */
	public function get_collection_params() {
		return array(
			'page'     => array(
				'description'        => 'Current page of the collection.',
				'type'               => 'integer',
				'default'            => 1,
				'sanitize_callback'  => 'absint',
			),
			'per_page' => array(
				'description'        => 'Maximum number of items to be returned in result set.',
				'type'               => 'integer',
				'default'            => 10,
				'sanitize_callback'  => 'absint',
			),
			'search'   => array(
				'description'        => 'Limit results to those matching a string.',
				'type'               => 'string',
				'sanitize_callback'  => 'sanitize_text_field',
			),
		);
	}


	/* ----------  OBJECTS  ---------- */

	/**
	 * Post data to webservice.
	 *
	 * @param string $url .
	 *
	 * @param string $args .
	 */
	private function post_data( $url, $args ) {
		$exc = $this->get_exc_default();

		$response = wp_remote_post( $url, $args );

		if ( is_wp_error( $response ) ) {
			$code = wp_remote_retrieve_response_code( $response );
			$errmsg = wp_remote_retrieve_response_message( $response );
			$exc = $this->get_exc( $code, $errmsg, null );
			return $exc;
		}

		$myex = $this->get_dataresult( $response );

		return $myex;
	}

	/**
	 * Get data from webservice.
	 *
	 * @param string $url .
	 */
	private function get_data( $url ) {
		$exc = $this->get_exc_default();
		$args = '';
		$response = wp_remote_get( $url, $args );
		if ( is_wp_error( $response ) ) {

			$code = wp_remote_retrieve_response_code( $response );
			$errmsg = wp_remote_retrieve_response_message( $response );

			$exc = $this->get_exc( $code, $errmsg, null );
			return $exc;
		}

		$myex = $this->get_dataresult( $response );

		return $myex;
	}


	/**
	 * Default exception.
	 */
	private function get_exc_default() {
		return array(
			'result'        => false,
			'errNumber'     => 0,
			'description'   => '',
			'data'          => array(),
		);
	}

	/**
	 * Default exception.
	 *
	 * @param int    $err_number  Error number.
	 *
	 * @param string $description Error message.
	 *
	 * @param array  $data        Response data.
	 */
	private function get_exc( $err_number, $description, $data ) {
		$res = array(
			'result'        => false,
			'errNumber'     => $err_number,
			'description'   => $description,
			'data'          => array(),
		);
		if ( $data && ! empty( $data ) ) {
			$res['data'] = $data;
		}
		return $res;
	}

	/**
	 * Get data result from OBEORN.
	 *
	 * @param object $response .
	 */
	private function get_dataresult( $response ) {
		$myex = $this->get_exc_default();

		$data = wp_remote_retrieve_body( $response );

		if ( empty( $data ) ) {
			$myex['result'] = false;
			$myex['errNumber'] = 99;
			$myex['description'] = 'Response error';
			return $myex;
		}

		$data = json_decode( $data );

		$myex['result'] = $data->result;
		$myex['errNumber'] = $data->errNumber;
		$myex['description'] = $data->description;
		$myex['data'] = $data->data;

		return $myex;
	}

	/**
	 * Handle webservice response error by status codes.
	 *
	 * @param int $code Response code.
	 */
	private function handle_response_error( $code, $errmsg ) {
		$exc = $this->get_exc_default();
		if ( 200 === $code) { // OK
			$exc['result'] = true;
		} else if ( 0 === $code ) {
			$exc = $this->get_exc( 111, 'The webservice server in unavailable.', null );
		} else if ( 404 === $code) {
			$exc = $this->get_exc( 404, 'Webservice method not resolved.', null );
		} else if ( 401 === $code) {
			$exc = $this->get_exc( 401, 'User not logged in.', null );
		} else if ( 500 === $code) {
			$exc = $this->get_exc( 500, 'Internal webservice server error.', null );
		// } else if (exception === 'parsererror') {
		// 	$exc = $this->get_exc( 900, 'Chyba pre spracovaní JSON dát zo servera.');
		// } else if (exception === 'timeout') {
		// 	$exc = $this->get_exc( 901, 'Čas pre spracovanie požiadavky vypršal.' );
		// } else if (exception === 'abort') {
		// 	$exc = $this->get_exc( 902, 'Asynchrónne volanie bolo prerušené.' );
		} else {
			$exc = $this->get_exc( $code, $errmsg, null );
		}
		return exc;
	}

}
