<?php
/**
 * OBERON_HotelReservation - Widget class.
 * 
 * This file is part of the OBERON System and WordPress plugins made by Exalogic company and is released under the same license.
 * For more information please see oberon_hotel_reservation_plugin.php.
 * 
 * Copyright (c) 2007-2020 Mário Moravcik. All rights reserved.
 * 
 * Version: 1.4.4
 */

class OBERON_HotelReservation extends WP_Widget {

	/**
	 * Create widget instance.
	 */
	function OBERON_HotelReservation() {
		if ( ! defined( 'ABSPATH' ) ) {
			return;
		}

		// Register needed resources.
		//add_action( 'wp_enqueue_scripts', array( $this, 'widget_scripts' ) );
		add_action( 'rest_api_init', array( $this, 'widget_rest_api' ) );

		if ( is_admin() ) {
			add_action( 'plugins_loaded', array( $this, 'load_textdomain' ) );
		}

		// load settings.
		$this->plugin_settings = get_option( OBERONRESERVATION_OPTION_NAME, array() );
		$this->reservation_type = 1;

		// add shortcode for pages with no widget areas.
		// echo '<!--- OHR - shortcode -->';
		//add_shortcode( 'oberon-hotel-reservation-form', 'widget_shortcode' );

		$widget_ops = array( 'description' => __( 'OBERON Hotel reservation form', OBERONRESERVATION_TEXT_DOMAIN ), );
		parent::WP_Widget( false, $name = 'OBERON Hotel Reservation', $widget_ops );
	}

	/** ----------------------------------------------------------------------------------------------- */
	/** ----------------------------------------------------------------------------------------------- */
	/** ------  W I D G E T   O V E R R I D E N  H A N D L E R S  -------------------------- */
	/** ----------------------------------------------------------------------------------------------- */
	/** ----------------------------------------------------------------------------------------------- */

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 */
	function update( $new_instance, $old_instance ) {
		$instance = array();

		// sanitize inputs
		$instance['reservation_type'] = ( ! empty( $new_instance['reservation_type'] ) ) ? strip_tags( $new_instance['reservation_type'] ) : 1;
		$this->validate_reservation_type( $this->reservation_type );
		$instance['reservation_type'] = $this->reservation_type;

		return $instance;
	}

	/**
	 * Back-end widget form.
	 *
	 * @param array $instance Previously saved values from database.
	 */
	function form( $instance ) {

		//$instance = wp_parse_args( (array) $instance, array('reservation_type'=> 1 ) );
		$restype = $this->validate_reservation_type( $instance['reservation_type'] );
		$url = $this->plugin_settings['service-url'];

		// Formulár pre úpravu priebežných nastavení.

		echo '<div>';
		echo '  <p class="description" style="padding-top: 1em;">';
		echo __( 'OBERON Web service connector version', OBERONRESERVATION_TEXT_DOMAIN ) . ': ' . $this->widget_version();

		echo '  <br />';

		if ( ! $url || empty( $url ) ) {
			echo __( 'Before using this plugin you have to set the OBERON Web url on plugin', OBERONRESERVATION_TEXT_DOMAIN );
			echo '&nbsp;';
			echo $this->my_settings_link( __( 'settings page', OBERONRESERVATION_TEXT_DOMAIN ) );
		} else {
			echo __( 'Webservice url:', OBERONRESERVATION_TEXT_DOMAIN );
			echo '&nbsp;&nbsp;' . esc_url( $url );

			echo '</p><p class="description" style="padding-top: 1em;">';
			echo '<select class="" id="' . esc_attr( $this->get_field_id('reservation_type') ) . '" name="' . esc_attr( $this->get_field_name('reservation_type') ) . '">';
			echo '<option value="1"';
			if ( 1 === $restype ) { 
				echo ' selected="select"';
			}
			echo '>' . __( 'Enguiry - info about capacity and price', OBERONRESERVATION_TEXT_DOMAIN ) . '</option>';
			echo '   <option value="2"';
			if ( 2 === $restype ) {
				echo ' selected="select"';
			}
			echo '>' . __( 'Binding reservation', OBERONRESERVATION_TEXT_DOMAIN ) . '</option>';
			echo '</select>';
		}

		echo '</p></div>';
	}


	/**
	 * Front-end display of widget.
	 *
	 * @param array $args .
	 *
	 * @param array $instance .
	 */
	function widget( $args, $instance ) {

		$reservation_type = $this->validate_reservation_type( $instance['reservation_type'] );
		$languageCode = $this->validate_lang_code( $instance['lang'] );

		/*
			Widget rendered on page is translated via javascript. It is posible to download also translation files from OBERON.
			That's why here is no translation in html.
		*/
		$this->widget_scripts();
		$this->generate_html( $reservation_type, $languageCode );

	}

	/**
	 * Display of widget with shortcode 'oberon-hotel-reservation-form'.
	 *
	 * @param array $atts Attributes in short code.
	 */
	public function widget_shortcode( $atts ) {

		global $wp_widget_factory;

		extract( shortcode_atts( array(
			'reservation_type' => 1,
			'lang' => 'en',
		), $atts ) );

		$reservation_type = $atts['reservation_type'];
		$langCode = $atts['lang'];

		$reservation_type = $this->validate_reservation_type( $reservation_type );
		$langCode = $this->validate_lang_code( $langCode );

		$this->widget_scripts();
		$this->generate_html( $reservation_type, $langCode );

	}

	/**
	 * Front-end display of widget.
	 *
	 * @param int $reservation_type .
	 */
	function generate_html( $reservation_type, $langCode ) {

?>

	<div id="oberonbs">
		<div id="oberon-hotel-reservation" class="container oberon-wrapper">
			<div class="oberon-hotel-header">
				<h4 id="widget_header_reservation">Rezervácia</h4>
			</div>
			<div id="reservation-slider" class="carousel slide">
				<ol class="carousel-indicators">
					<li data-target="#carouselExampleIndicators" data-slide-to="0" class="active"></li>
					<li data-target="#carouselExampleIndicators" data-slide-to="1"></li>
					<li data-target="#carouselExampleIndicators" data-slide-to="2"></li>
				</ol>
				<div class="carousel-inner">
					<div class="carousel-item active" id="carousel-ohr-main">	
						<form id="oberon-hotel-reservation-main">
							<div class="form-row">										
								<div class="form-group col-md-4">
									<label id="l_t_date_from" class="control-label" for="t_date_from">Dátum od</label>
									<div class="input-group mb-0 mb-sm-0">
										<div class="input-group-addon"><i class="icon-calendar"></i></div>
										<input type="text" class="form-control" id="t_date_from" name="t_date_from" />
									</div>	
								</div>
								<div class="form-group col-md-4">
									<label id="l_t_date_to" class="control-label" for="t_date_to">Dátum do</label>
									<div class="input-group mb-0 mb-sm-0">
										<div class="input-group-addon"><i class="icon-calendar"></i></div>
										<input type="text" class="form-control" id="t_date_to" name="t_date_to" />
									</div>	
								</div>											
								<div class="form-group col-md">
									<label id="l_t_rooms" class="control-label" for="t_rooms">Počet izieb</label>
									<div class="input-group mb-0 mb-sm-0">
										<select  class="form-control" id="t_rooms" name="t_rooms">
											<option value="1" selected="selected">1 izba</option><option value="2">2 izby</option><option value="3">3 izby</option><option value="4">4 izby</option><option value="5">5 izieb</option>
										</select>
									</div>
								</div>
							</div>
							<div id="room_list">
								<!-- Room list changed on room count selection -->							
							</div>
							<div class="form-row">
								<div class="form-group col-md">
									<label id="l_t_submission_type" class="control-label" for="t_submission_type">Spôsob spracovania údajov</label>
									<div class="input-group mb-2 mb-sm-0">
										<select  class="form-control" id="t_submission_type" name="t_submission_type">
										</select>
									</div>	
								</div>		
							</div>
							<div class="form-row">
								<div class="form-group col-md">
									<label id="l_t_notice" for="t_notice">Doplňujúce informácie</label>
									<div class="input-group mb-2 mb-sm-0">
										<textarea rows="5" cols="20" class="form-control" id="t_notice" name="t_notice"></textarea>
									</div>
								</div>	
							</div>						
						</form>
					</div>
					<div class="carousel-item" id="carousel-ohr-verify">	
						<div style="margin: 30px 0;line-height: 18px;" id="l_slide2_text">
							Pred odoslaním záväznej rezervácie je možné overiť dostupnosť voľných kapacít a zároveň vytvoriť cenovú kalkuláciu.
						</div>
						<div class="oberon-hotel-calculation">
							<input type="hidden" value="" id="t_calculatedprice" name="t_calculatedprice" />							
							<button type="submit" id="btn_calculation" class="btn btn-tertiary pull-right">Overiť dostupnosť</button>
						</div>
						<div id="calculation-result" class="alert">							
						</div>
						<div id="calculation-price-table" class="price-table-result" style="display: none">
							<table class="table">
								<thead class="thead-default">
									<tr style="background-color: #f5f5f5;">
										<th id="pricetab_name">Popis</th>
										<th id="pricetab_unitprice">Jedn. cena</th>
										<th id="pricetab_amount">Množstvo</th>
										<th id="pricetab_unit">Jednotka</th>
										<th id="pricetab_price">Cena celkom</th>
									</tr>
								</thead>
								<tbody id="calculation-price-table-items"></tbody>
							</table>							
						</div>
						<div class="oberon-wizard-spacer"></div>
					</div>
					<div class="carousel-item" id="carousel-ohr-data">
						<form id="oberon-hotel-reservation-main">
							<div class="form-row">										
								<div class="form-group col-md-5">
									<label class="control-label" for="t_name">Meno</label>
									<div class="input-group mb-0 mb-sm-0">
										<input type="text" class="form-control" id="t_name" name="t_name"  placeholder="given name" />
									</div>	
								</div>
								<div class="col-md"></div>
								<div class="form-group col-md-5">
									<label class="control-label" for="t_namefamily">Priezvisko</label>
									<div class="input-group mb-0 mb-sm-0">
										<input type="text" class="form-control" id="t_namefamily" name="t_namefamily" placeholder="family name" />
									</div>	
								</div>																			
							</div>	

							<div class="form-row">										
								<div class="form-group col-md-5">
									<label class="control-label" for="t_email">E-mail:</label>
									<div class="input-group mb-0 mb-sm-0">
										<input type="text" class="form-control" id="t_email" name="t_email"  placeholder="email" />
									</div>	
								</div>
								<div class="col-md"></div>
								<div class="form-group col-md-5">
									<label class="control-label" for="t_phone">Telefón:</label>
									<div class="input-group mb-0 mb-sm-0">
										<input type="text" class="form-control" id="t_phone" name="t_phone" placeholder="phone number" />
									</div>	
								</div>																			
							</div>

							<div class="form-row">										
								<div class="form-group col-md-5">
									<label class="control-label" for="t_street">Ulica</label>
									<div class="input-group mb-0 mb-sm-0">
										<input type="text" class="form-control" id="t_street" name="t_street"  placeholder="street" />
									</div>	
								</div>
								<div class="col-md"></div>
								<div class="form-group col-md-5">
									<label class="control-label" for="t_city">Obec</label>
									<div class="input-group mb-0 mb-sm-0">
										<input type="text" class="form-control" id="t_city" name="t_city" placeholder="town or city" />
									</div>	
								</div>																			
							</div>

							<div class="form-row">										
								<div class="form-group col-md-5">
									<label class="control-label" for="t_postalcode">PSČ</label>
									<div class="input-group mb-0 mb-sm-0">
										<input type="text" class="form-control" id="t_postalcode" name="t_postalcode"  placeholder="postal code" />
									</div>	
								</div>
								<div class="col-md"></div>
								<div class="form-group col-md-5">
									<label class="control-label" for="t_country">Krajina</label>
									<div class="input-group mb-0 mb-sm-0">
										<input type="text" class="form-control" id="t_country" name="t_country" placeholder="country" />
									</div>	
								</div>																			
							</div>

						</form>						
						<div class="oberon-wizard-spacer"></div>
					</div>
					<div class="carousel-item" id="carousel-ohr-result">
						<div id="oberon-reservation-result" class="alert">
						</div>
						<div class="oberon-wizard-page-buttons">
							<!--
							<button type="submit" id="btn_new_reservation" class="btn btn-tertiary pull-right">Nová rezervácia</button>
							-->
						</div>
						<div class="oberon-wizard-spacer"></div>
					</div>
					<!-- Error page -->
					<div class="carousel-item" id="carousel-ohr-err">
						<div id="oberon-reservation-error" class="alert">
							<div class="oberon-error-icon"></div>
							<div class="oberon-error-message"></div>
						</div>
						<div class="oberon-wizard-spacer"></div>
					</div>
				</div>
			</div>
			<div class="form-actions">
				<button type="submit" id="btn_prev" class="btn btn-secondary">Naspäť</button>
				<button type="submit" id="btn_next" class="btn btn-secondary">Ďalej</button>
				<button type="submit" id="sendRequest" class="btn btn-primary" disabled>Odoslať</button>
			</div>
		</div>
	</div>
	<script type="text/javascript">
		var oberonHotelReservationSettings = {
				baseUrl: "<?php echo  esc_url( OBERONRESERVATION_URL ); ?>",
				preferedSubmissionType: <?php echo esc_attr( $reservation_type ); ?>,
				lang: "<?php echo esc_attr( $langCode ); ?>"
			}
	</script>
<?php
	}
	/**
	 * Get the OBERON Web widget version property.
	 *
	 * @param string $format .
	 */
	public function widget_version( $format = true ) {
		$ver = OBERONRESERVATION_VERSION;
		if ( $format ) {
			$ver = substr( number_format( $ver, 0, '', '.' ), 0, 3 );
		}
		return $ver;
	}

	/**
	 * Load widget text domain.
	 */
	public function load_textdomain() {
		$locale = apply_filters( 'plugin_locale', get_locale(), OBERONRESERVATION_TEXT_DOMAIN );
		//load_plugin_textdomain( OBERONRESERVATION_TEXT_DOMAIN, false, basename( dirname( __FILE__ ) ) . '/languages' );
		load_plugin_textdomain( OBERONRESERVATION_TEXT_DOMAIN, false, basename( dirname( __FILE__ ) ) . '/languages' );
	}

	/**
	 * Get link to edit settings.
	 *
	 * @param string $text .
	 * @param array  $links .
	 */
	public function my_settings_link( $text, $links = null ) {
		$settings_link = sprintf( '<a href="%s">%s</a>', admin_url( 'options-general.php?page=exalogic-oberon-hotel-reservation' ), $text );
		if ( $links && ! empty( $links ) ) {
			array_unshift( $links, $settings_link );
		}
		return $settings_link;
	}

	/**
	 * Register admin scripts.
	 */
	function widget_scripts() {

		wp_enqueue_script( 'oberon-application-base', OBERONRESERVATION_URL . 'js/exalogic-oberon-application-base.js', array( 'jquery', 'jquery-ui-core', 'jquery-ui-datepicker' ), '1.0', true );
		wp_enqueue_script( 'jquery-lingua', OBERONRESERVATION_URL . 'js/jquery.lingua.min.js', array( 'jquery' ), '1.0', true );
		wp_enqueue_script( 'bootstrap-popper', OBERONRESERVATION_URL . 'js/popper.min.js', array( 'jquery' ), '', true );
		wp_enqueue_script( 'bootstrap', OBERONRESERVATION_URL . 'js/bootstrap.min.js', array( 'bootstrap-popper' ), '4.0', true );
		wp_enqueue_script( 'bootstrap-util', OBERONRESERVATION_URL . 'js/bootstrap-util.js', array( 'bootstrap' ), '4.0', true );
		wp_enqueue_script( 'bootstrap-carousel', OBERONRESERVATION_URL . 'js/bootstrap-carousel.js', array( 'bootstrap' ), '4.0', true );		
		wp_enqueue_script( 'oberon-hotelreservation-js', OBERONRESERVATION_URL . 'js/exalogic-oberon-hotel-reservation.js', array( ), '1.0', true );
		wp_enqueue_script( 'exalogic-oberon-application-widget-js', OBERONRESERVATION_URL . 'js/exalogic-oberon-application-widget.js', array( ), '1.0', true );
		
		wp_enqueue_style( 'oberon-vendor-styles', OBERONRESERVATION_URL . 'css/exalogic-oberon-vendor-styles.css', array(), '1.0' );
		wp_enqueue_style( 'oberon-fontello-hotel', OBERONRESERVATION_URL . 'css/fontello-hotel.css', array(), '1.0' );
		wp_enqueue_style( 'jquery-ui-1-10', OBERONRESERVATION_URL . 'css/jquery-ui-1.10.0.custom.min.css', array(), '1.0' );
		wp_enqueue_style( 'oberon-styles', OBERONRESERVATION_URL . 'css/exalogic-oberon-styles.css', array(), '1.0' );

	}

	/**
	 * Register JSON REST APIs.
	 */
	function widget_rest_api() {
		$controller = new OBERON_HotelReservation_RouteController();
		$controller->register_routes();
	}

	/**
	 * Validate user input on reservation type.
	 *
	 * @param int $reservation_type .
	 */
	protected function validate_reservation_type( $reservation_type ) {
		if ( ! $reservation_type ) {
			return 1;
		}
		$reservation_type = absint( $reservation_type );
		if ( $reservation_type < 1 || $reservation_type > 2 ) {
			$reservation_type = 1;
		}
		return $reservation_type;
	}

	/**
	 * Validate user input on language code.
	 *
	 * @param string $langCode ISO two character language code (eg. 'en', 'de', 'sk',...).
	 */
	protected function validate_lang_code( $langCode ) {
		$res = $langCode;
		if ( ! $res ) {
			return 'sk';
		}
		$res = strtolower( $res );
		$res = trim($res, " \t\n\r");
		$len = strlen($res);
		if( $len > 2 ) {
			$res = substr( res, 0, 2 );
		} elseif ( $len < 2 ) {
			return 'sk';	
		}
		return $res;
	}

}

?>
