<?php
/**
 * OBERON_HotelReservation - Widget class.
 * 
 * This file is part of the OBERON System and WordPress plugins made by Exalogic company and is released under the same license.
 * For more information please see oberon_hotel_reservation_plugin.php.
 * 
 * Copyright (c) 2007-2018 Mário Moravcik. All rights reserved.
 * 
 * Version: 1.4.4
 */

if ( ! defined( 'OBERONRESERVATION_ADMIN_PAGE' ) ) {
	exit;
}

if ( isset( $_POST['oberon-settings-save'] ) ) {
	check_admin_referer( 'oberon-settings-save-order' );
	$this->save_settings();
}

$usrid = get_current_user_id();
$lang = get_user_locale( $usrid );
$lang = substr( $lang, 0, 2 );
if ( !$lang ) {
	$lang = 'en';
}
?>

<div id="oberon-settings" class="wrapper">
	<div class="message-container"></div>
	<h2><?php _e( 'OBERON Hotel Reservation', OBERONRESERVATION_TEXT_DOMAIN ); ?></h2>
	<div class="description">
		<?php _e( 'This plugin serves as a front-desk hotel accommodation reservation form for Hotel module in system OBERON. Plugin uses OBERON Web api online services for communication with hotel system to create reservations, check for room availability or price.', OBERONRESERVATION_TEXT_DOMAIN ); ?>
	</div>
	<div class="area">
		<h3><?php _e( 'Glossary', OBERONRESERVATION_TEXT_DOMAIN ); ?>:</h3>		
		<h4>OBERON Center</h4>
		<div class="description">			
			<?php _e( 'OBERON Center is part of system OBERON package and used for background tasks. Part of this is running webservices servers used for online communication.', OBERONRESERVATION_TEXT_DOMAIN ); ?>
		</div>
		<h4>OBERON Web</h4>
		<div class="description">
		<?php _e( 'OBERON Web is online webservice type for communication between system OBERON and its web components or third party services.', OBERONRESERVATION_TEXT_DOMAIN ); ?>
		</div>

		<h4><?php _e( 'How to', OBERONRESERVATION_TEXT_DOMAIN ); ?>:</h4>
		<ol>
			<li><?php _e( 'The main thing is to ensure that the webservice is availabe online for internet access. This is done with public ip and port configured by company ISP (internet services provider).', OBERONRESERVATION_TEXT_DOMAIN ); ?></li>
			<li><?php _e( 'The second major issue is to configure internal network and firewalls to allow access from internet to specific port used by OBERON Web service for communication. This should be in competence of company internal network administrator.', OBERONRESERVATION_TEXT_DOMAIN ); ?></li>
			<li><?php 
				_e( 'Install (if not already) new </em>OBERON Web</em> service. Full documentation in slovak language is available in', OBERONRESERVATION_TEXT_DOMAIN ); ?>
				<a href="https://exalogic.sk/help/oberon/index.html#Developers_WebServices_Installation.html" target="_blank"> <?php _e( 'online help', OBERONRESERVATION_TEXT_DOMAIN ); ?> </a></li>

		</ol>
		<h4><?php _e( 'Add to page', OBERONRESERVATION_TEXT_DOMAIN ); ?>:</h4>
		<ol>
			<li><?php _e( 'On widgets page you can add the plugin to panel or widget page area.', OBERONRESERVATION_TEXT_DOMAIN ); ?></li>
			<li><?php echo sprintf( __( 'Use shortcode <em>%s</em> in page text.', OBERONRESERVATION_TEXT_DOMAIN ), '[' . OBERONRESERVATION_SHORTCODE . ']' ); ?></li>
		</ol>
	</div>

	<div class="area">
		<h3><?php _e( 'Settings', OBERONRESERVATION_TEXT_DOMAIN ); ?>:</h3>
		<div><?php _e( 'OBERON Web service connector version', OBERONRESERVATION_TEXT_DOMAIN ); ?>: <?php echo $this->widget_version(); ?>.</div>
		<form id="oberon-settings-admin" action="" method="post">						
			<div class="control">
				<label for="oberon-web-url">OBERON Web - <?php _e( 'webservice url', OBERONRESERVATION_TEXT_DOMAIN ); ?>:</label>
				<input type="text" id="oberon-web-url" name="oberon-web-url" value="<?php echo esc_url( $this->get_oberon_web_url() ); ?>" />
				<input type="button" id="oberon-test-connection" name="oberon-test-connection"  class="button-primary button-large <?php if(!$this->get_oberon_web_url()) { echo 'disabled'; } ?>" value="<?php _e( 'Test connection', OBERONRESERVATION_TEXT_DOMAIN ); ?>" />
			</div>
			<h4><?php _e( 'Reservation form', OBERONRESERVATION_TEXT_DOMAIN ); ?></h4>
			<div class="description">				
				<div class="control">				
					<input type="checkbox" id="allow-roomtype-change" name="allow-roomtype-change" value="1" <?php checked( 1, $this->get_allow_roomtype_change(), true ); ?>/>
					<label for="allow-roomtype-change"><?php _e( 'Allow system to automatically change room type on reservation availability check, when desired room type is unavailable', OBERONRESERVATION_TEXT_DOMAIN ); ?>.</label>					
					<div class="description"><?php _e( 'Example: when desired room type 2+0 is unavailable, the system can use 2+1 (when 2+1 room type is free)', OBERONRESERVATION_TEXT_DOMAIN ); ?>.</div>
				</div>
				<div class="control">				
					<input type="checkbox" id="calculate-accommodation" name="calculate-accommodation" value="1" <?php checked( 1, $this->get_reservation_calc_accommodation(), true ); ?>/>
					<label for="calculate-accommodation"><?php _e( 'Calculate accommodation price', OBERONRESERVATION_TEXT_DOMAIN ); ?>.</label>
				</div>
				<div class="control">				
					<label for="price-category-guest"><?php _e( 'Guest price category', OBERONRESERVATION_TEXT_DOMAIN ); ?>:</label>
					<select id="price-category-guest" name="price-category-guest">
					<?php
					if ( $this->get_price_category_guest() ) {
						echo '<option value="' . $this->get_price_category_guest() . '" selected="selected">'. $this->get_price_category_guest() . '</option>';
					}
					?>
					</select>
					<div class="description"><?php _e( 'Guest category is used to calculate price for accommodation. When left blank, default category will be used', OBERONRESERVATION_TEXT_DOMAIN ); ?>.</div>
				</div>												
				<!--<div class="control">				
					<input type="checkbox" disabled="disabled" id="calculate-localtax" name="calculate-localtax" value="1" <?php checked( 1, $this->get_reservation_calc_localtax(), true ); ?>/>
					<label for="calculate-localtax">< ?php _e( 'Calculate local tax', OBERONRESERVATION_TEXT_DOMAIN ); ?>.</label>
				</div>
				<div class="control">				
					<input type="checkbox" disabled="disabled" id="calculate-services" name="calculate-services" value="1" <?php checked( 1, $this->get_reservation_calc_services(), true ); ?>/>
					<label for="calculate-services">< ?php _e( 'Calculate basic services', OBERONRESERVATION_TEXT_DOMAIN ); ?>.</label>					
				</div>-->					
			</div>
			<div class="buttons">
				<?php wp_nonce_field( 'oberon-settings-save-order' ); ?>
				<input class="button-primary button-large" type="submit" name="oberon-settings-save" value="<?php _e( 'Save Changes', OBERONRESERVATION_TEXT_DOMAIN ); ?>" />	
			</div>
		</form>
	</div>
	
	<div class="orpa-settings-message" style="display: none;"><!-- only  -->		
		<div class="message-header"></div>
		<div class="message-body">
			<div class="message-row">
				<div class="inner-icon"></div>
				<div class="inner-message">	
				</div>
			</div>
			<div class="button-row"></div>
		</div>
	</div>

</div>

<script type="text/javascript">

(function () {
	jQuery( document ).ready(function(){
		
		var lang = '<?php echo esc_attr( $lang ); ?>';
		jQuery.linguaInit("<?php echo  esc_url( OBERONRESERVATION_URL ); ?>lang/", "settings-page");
		jQuery.linguaLoadAutoUpdate(lang);
		
		jQuery('#oberon-settings').rpAdminPage({ root:'<?php echo esc_url_raw( rest_url() ) ?>', nonce: '<?php echo wp_create_nonce( 'wp_rest' ) ?>' });

	});
}).call(this);

</script>
