﻿/**
 * OBERONApplicationBase - Base methods used in OBERON webservices javascripts.
 * 
 * This file is part of the OBERON System and WordPress plugins made by Exalogic company and is released under the same license.
 * For more information please see oberon_hotel_reservation_plugin.php.
 * 
 * Copyright (c) 2007-2017 Mário Moravcik. All rights reserved.
 * 
 * Version: 1.4.4
 */
(function () {
    var $ = jQuery;
    var OBERONApplicationBase = function () {
        return new OBERONApplicationBase.init();
    };    
    OBERONApplicationBase.settings = {
        version: "1.0",
        monthNames: ['Január', 'Február', 'Marec', 'Apríl', 'Máj', 'Jún', 'Júl', 'August', 'September', 'Október', 'November', 'December'],
        dayNames: ['Nedeľa', 'Pondelok', 'Utorok', 'Streda', 'Štvrtok', 'Piatok', 'Sobota'],
        dayNamesShort: ['ne', 'po', 'ut', 'st', 'št', 'pi', 'so'], 
		apiNonce: '',        
    };        
    OBERONApplicationBase.init = function () {        
    };
	OBERONApplicationBase.postData = function (ajaxurl, jdata, rfResult) {
        var exc = OBERONApplicationBase.myException('');
        $.ajax({ type: 'POST', url: ajaxurl, cache: false, async: false,
            dataType: 'json', contentType: 'application/json; charset=utf-8',
            processData: false,
            data: JSON.stringify(jdata), //--- musí byt konverzia na text
            beforeSend: function (xhr) {
				if ( OBERONApplicationBase.settings.apiNonce && OBERONApplicationBase.settings.apiNonce.length != 0 ) {				
					xhr.setRequestHeader( 'X-WP-Nonce', OBERONApplicationBase.settings.apiNonce );
				}
            },
            success: function (data, textStatus, xhr) {
				dataResultGet(data, rfResult);              
                exc.result = true;//--- default je true
                if (rfResult.result == false) {
                    exc.result = false;
                    exc.errNumber = rfResult.errNumber;
                    exc.description = rfResult.description;
                }
            },
            error: function (xhr, exception, errorThrown) {
                exc = z_GetAjaxException(xhr, exception, errorThrown);
                OBERONApplicationBase.logException(exc);
            }
        });
        return exc;
    };
    OBERONApplicationBase.getData = function (ajaxurl, rfResult, jdata, processdata) {
        var exc = OBERONApplicationBase.myException('');
        if (!processdata) { processdata = false; }
        if (!jdata) { jdata = null; }
        $.ajax({ type: 'GET', cache: false, async: false,
            dataType: 'json', contentType: 'application/json; charset=utf-8',
            url: ajaxurl, processData: processdata,
            data: JSON.stringify(jdata), //--- musí byt konverzia na text,
            beforeSend: function (xhr) {            
				if ( OBERONApplicationBase.settings.apiNonce && OBERONApplicationBase.settings.apiNonce.length != 0 ) {
					xhr.setRequestHeader( 'X-WP-Nonce', OBERONApplicationBase.settings.apiNonce );
				}
            },
            success: function (data, textStatus, xhr) {
                dataResultGet(data, rfResult);
                exc.result = true; //--- default is true
                if (rfResult.result == false) {
                    exc.result = false;
                    exc.errNumber = rfResult.errNumber;
                    exc.description = rfResult.description;
                }                
            },
            error: function (xhr, exception, errorThrown) {
                exc = z_GetAjaxException(xhr, exception, errorThrown);
                OBERONApplicationBase.logException(exc);
            }
        });
        return exc;
    };
    OBERONApplicationBase.getHtml = function (ajaxurl, rfResult, callType, jdata) {
        var exc = OBERONApplicationBase.myException('');
        if (!callType) { callType = 'GET'; }
        $.ajax({ type: callType, url: ajaxurl, cache: false, async: false, timeout: 10000,
            dataType: 'html', contentType: 'text/html; charset=utf-8',
            processData: false, data: JSON.stringify(jdata),
            beforeSend: function (xhr) {
				if ( OBERONApplicationBase.settings.apiNonce && OBERONApplicationBase.settings.apiNonce.length != 0 ) {
					xhr.setRequestHeader( 'X-WP-Nonce', OBERONApplicationBase.settings.apiNonce );
				}
            },
            success: function (data, textStatus, xhr) {
                rfResult.html = data;
                exc.result = true;
            },
            error: function (xhr, exception, errorThrown) {
                exc = z_GetAjaxException(xhr, exception, errorThrown);
                OBERONApplicationBase.logException(exc);
            }
        });
        return exc;
    };    
    OBERONApplicationBase.numFormat = function (num, decimals, useFormat) {
        var res = '';
        if (useFormat && useFormat != true) {
            return eval(num).toFixed(eval(decimals));
        } else {
            return z_numFormat(num, decimals, ',', ' ');            
        }
    };
	//--- Format number that is always in correct format 12345.45.
    function z_numFormat(number, decimals, dec_point, thousands_sep) {
        number = (number + '').replace(/[^0-9+\-Ee.]/g, '');
        var n = !isFinite(+number) ? 0 : +number,
          prec = !isFinite(+decimals) ? 0 : Math.abs(decimals),
          sep = (typeof thousands_sep === 'undefined') ? ',' : thousands_sep,
          dec = (typeof dec_point === 'undefined') ? '.' : dec_point,
          s = '',
          toFixedFix = function (n, prec) {
              var k = Math.pow(10, prec);
              return '' + (Math.round(n * k) / k).toFixed(prec);
          };
        // Fix for IE parseFloat(0.55).toFixed(0) = 0;
        s = (prec ? toFixedFix(n, prec) : '' + Math.round(n)).split('.');
        if (s[0].length > 3) {
            s[0] = s[0].replace(/\B(?=(?:\d{3})+(?!\d))/g, sep);
        }
        if ((s[1] || '').length < prec) {
            s[1] = s[1] || '';
            s[1] += new Array(prec - s[1].length + 1).join('0');
        }
        return s.join(dec);
    };
	//--- Format (get) date from .NET webservice value structure to number in js.
    OBERONApplicationBase.dateGet = function (dat) {
        var result = null;
        try {
            result = new Date(dat);
            if (typeof (result) === "date") { return result; }
        } catch (e) {/* testujeme ešte nižšie */}
        if (typeof (dat) === "string") {
            dat = $.trim(dat);
            if (dat.indexOf('/Date(') >= 0) {
                result = new Date(parseFloat(dat.replace("/Date(", "").replace(")/", "")));
            } else if (dat.indexOf('-') > -1) {
                if (dat.indexOf(':') > -1) {
                    //obsahuje dátum a čas
                    result = new Date(dat.replace(/(\d{2,4})\-\s*(\d{1,2})\-\s*(\d{1,2})\s*(\d{1,2})\:\s*(\d{1,2})\:\s*(\d{1,2})/, "$1-$2-$3T$4:$5:00"));
                } else {
                    result = new Date(dat.replace(/(\d{2,4})\-\s*(\d{1,2})\-\s*(\d{1,2})/, "$2/$3/$1"));
                }
            } else if (dat.indexOf('.') > -1) {
                if (dat.indexOf(':') > -1) {
                    //obsahuje dátum a čas dd.mm.yyyy hh:mm:ss
                    result = new Date(dat.replace(/(\d{2})\.(\d{2})\.(\d{4})\s+(\d{2})\:(\d{2})\:(\d{2})/, "$3-$2-$1T$4:$5:00"));
                } else {
                    result = new Date(dat.replace(/(\d{2})\.(\d{2})\.(\d{4})/, "$2/$1/$3"));
                }
            }
            if (result) {
                if (result < new Date("1/1/1900")) { result = null; }
            }
        } else { result = dat; }
        return result;
    };
	//--- Format date to expected format dd.MM.yyyy.
    OBERONApplicationBase.dateFormat = function (dat, format) {
        var date;
        if (!dat) { return ''; }
        if (dat instanceof Date) { date = dat; } else { date = OBERONApplicationBase.dateGet(dat); }
        if (!date) { return ''; }
        if (!format) { format = 'dd.MM.yyyy'; }
        var month = date.getMonth() + 1;
        var year = date.getFullYear();
        format = format.replace("MM", month.toString().padL(2, "0"));
        if (format.indexOf("yyyy") > -1) { format = format.replace("yyyy", year.toString()); }
        if (format.indexOf("yy") > -1) { format = format.replace("yy", year.toString().substr(2, 2)); }
        format = format.replace("dd", date.getDate().toString().padL(2, "0"));
        var hours = date.getHours();
        if (format.indexOf("HH") > -1) { format = format.replace("HH", hours.toString().padL(2, "0")); }
        if (format.indexOf("t") > -1) {
            //--- ak je 12 hodinový formát tak musí byť označenie pm/am
            if (hours > 11) { format = format.replace("t", "pm"); } else { format = format.replace("t", "am"); }
            if (format.indexOf("hh") > -1) {
                if (hours > 12) { hours -= 12; }
                if (hours === 0) { hours = 12; }
                format = format.replace("hh", hours.toString().padL(2, "0"));
            }
        }
        if (format.indexOf("mm") > -1) { format = format.replace("mm", date.getMinutes().toString().padL(2, '0')); }
        if (format.indexOf("ss") > -1) { format = format.replace("ss", date.getSeconds().toString().padL(2, '0')); }
        return format;
    };
    OBERONApplicationBase.monthName = function (dat, getShort) {
        if (!dat) { return ''; }
        if (dat instanceof Date) { date = dat; } else { date = OBERONApplicationBase.dateGet(dat); }
        if (!date) { return ''; }
        var res = OBERONApplicationBase.settings.monthNames[date.getMonth()];
        if (getShort && getShort == true) { res = res.substr(0, 3); }
        return res;
    };
    OBERONApplicationBase.dayName = function (dat, getShort) {
        if (!dat) { return ''; }
        if (dat instanceof Date) { date = dat; } else { date = OBERONApplicationBase.dateGet(dat); }
        if (!date) { return ''; }
        var res = '';
        if (getShort && getShort == true) {
            res = OBERONApplicationBase.settings.dayNamesShort[date.getDay()];
        } else {
            res = OBERONApplicationBase.settings.dayNames[date.getDay()];
        }
        return res;
    };
    OBERONApplicationBase.pause = function (milisec) {
        var date = new Date();
        var curDate = null;
        do { curDate = new Date(); }
        while (curDate - date < milisec);
    };
    OBERONApplicationBase.wait = function (ms, handler) {
        setTimeout(handler, ms);
    };
	// Redirect page.
    OBERONApplicationBase.pageRedirect = function (url) { document.location.href = url; };
	// Application type exceptions.
    OBERONApplicationBase.myException = function (descr, msg, errNum, res) { //exception object
        return { result: !res ? false : res, errNumber: !errNum ? 0 : errNum, message: !msg ? '' : msg, description: !descr ? '' : descr };
    };
	// Get empty ajax call data result object for reference.
    OBERONApplicationBase.dataResult = function () { //data result object
        return { result: false, errNumber: 0, description: '', data: null };
	};
	// Get empty method result.
    OBERONApplicationBase.result = function () {
        this.errNumber = 0;
        this.description = '';
        this.result = false;
    };
	// Log error to browser console with message.
    OBERONApplicationBase.log = function (msg) {
        if (!msg) return;
        setTimeout(function () {
            throw new Error(msg);
        }, 0);
	};
	// Log method exception.
    OBERONApplicationBase.logException = function (exc) {
        if (!exc) return;
        var msg = '';
        msg += !exc.errNumber ? '' : exc.errNumber + ': ';
        msg += !exc.description ? '' : exc.description;
        msg += !exc.message ? '' : ' (' + exc.message + ')';
        OBERONApplicationBase.log(msg);
    };
    OBERONApplicationBase.validateEmail = function (email) {
        return OBERONApplicationBase.validateRegEx(email, /^(([^<>()\[\]\\.,;:\s@\"]+(\.[^<>()\[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/);
    };
    OBERONApplicationBase.validatePhone = function (phoneNo) {
        return OBERONApplicationBase.validateRegEx(phoneNo, /^([\+]{1}[\d]{3}?|[\d]{5}|[\d]{4})?[\s\.-]*(\d{3})[\s\.-]*(\d{3})[\s\.-]*(\d{3})?$/);
    };
    OBERONApplicationBase.validateIP = function (ip) {
        return OBERONApplicationBase.validateRegEx(ip, /^\d{1,3}.{0,1}\d{1,3}.{0,1}\d{1,3}.{0,1}\d{1,3}$/);
    };
    OBERONApplicationBase.validateRegEx = function (text, regex) {
        if (!text || text.length === 0) { return false; }
        if (!regex) { return false; }
        var res = regex.test(text);
        return res;
    };
    // ovládanie okna pre čakanie na dokončenie úlohy
    OBERONApplicationBase.waitShow = function () {
        if (OBERONApplicationBase.settings.isWaiting == true) { return false; }
        OBERONApplicationBase.waitMessage();
        $('#wndWait').modal('show');
        OBERONApplicationBase.settings.isWaiting = true;
    };
    OBERONApplicationBase.waitToggle = function (wait) {
        OBERONApplicationBase.waitMessage();
        var toDo = '';
        if (OBERONApplicationBase.settings.isWaiting == false) { toDo = 'show' } else { toDo = 'hide' }
        $('#wndWait .modal-footer').hide();
        if (!wait) {
            //$('#wndWait').modal('toggle');
            $('#wndWait').modal(toDo);
            if (toDo == 'show') { OBERONApplicationBase.settings.isWaiting = true } else { OBERONApplicationBase.settings.isWaiting = false }
        } else {
            setTimeout(function () {
                $('#wndWait').modal(toDo);
                if (toDo == 'show') { OBERONApplicationBase.settings.isWaiting = true } else { OBERONApplicationBase.settings.isWaiting = false }
            }, wait);
        }
    };
    OBERONApplicationBase.waitMessage = function (msg, icon) {
        if (!msg) {
            $('#wndWait .msg').empty();
            $('#wndWait .msg').hide();
        } else {
            $('#wndWait .msg').html(msg);
            $('#wndWait .msg').show();
        }
        if (!icon) { icon = 'loader' }
        if (icon) {
            var elm = $('#wndWait .wait-icon');
            elm.removeClass();//odstrániť všetky classy
            elm.addClass('wait-icon');
            switch (icon) {
                case 'w': case 'warn': case 'warning':
                    elm.addClass('msg-alert');
                    break;
                case 'n': case 'noacc': case 'noaccess':
                    elm.addClass('msg-noaccess');
                    break;
                case 'i': case 'inf': case 'info':
                    elm.addClass('msg-info');
                    break;
                case 'e': case 'err': case 'error':
                    elm.addClass('msg-error');
                    break;
                case 'o': case 'ok': case 'success': case 'confirm':
                    elm.addClass('msg-confirm');
                    break;
                default:
                    elm.addClass('loader');
                    break;
            }        
        }
    };
    OBERONApplicationBase.waitMessageEx = function (exc, timeout, icon) {
        if (!exc) {
            OBERONApplicationBase.waitMessage('Neznáma chyba.', 'e');
        } else {
            if(exc.result == true) {
                msg = '';
                icon = '0';
            } else {
                var msg = exc.errNumber != 0 ? 'Číslo chyby: ' + exc.errNumber + '<br />' : '';
                msg = exc.description;
                if (!icon) { icon = 'e'; /* prebiť ikonu */ }
                if (exc.errNumber === 9) { icon = 'u'; /* unauthorized */ }                
            }
            OBERONApplicationBase.waitMessage(msg, icon);
        }
        if (timeout && timeout != 0) {
            OBERONApplicationBase.waitClose(timeout);
        }
    }
    OBERONApplicationBase.waitMessageHtml = function (elm, icon, showControls) {
        if (!elm) {
            OBERONApplicationBase.waitMessage('Neznáma chyba.', 'e');
        } else {
            OBERONApplicationBase.waitMessage(elm, icon);
        }
        if (showControls) {
            $('#wndWait .modal-footer').show();
            $('#wndWait .modal-footer .wnd-close').on('click', function () {
                OBERONApplicationBase.waitClose();
            });
        } else {
            $('#wndWait .modal-footer').hide();
            ('#wndWait .modal-footer .wnd-close').off('click');
        }
    }
    OBERONApplicationBase.waitClose = function (timeout) {
        if (OBERONApplicationBase.settings.isWaiting == false) { return false; }
        if (!timeout || !$.isNumeric(timeout)) {
            $('#wndWait').modal('hide');
            OBERONApplicationBase.settings.isWaiting = false
        } else {
            setTimeout(function () { $('#wndWait').modal('hide'); OBERONApplicationBase.settings.isWaiting = false; }, timeout);
        }
    };    
	/* private methods */	
    function dataResultGet(data, rfResult) {//rfResult musí byť inicializovaný
        rfResult.result = data.result;
        rfResult.errNumber = data.errNumber;
        rfResult.description = data.description;
        rfResult.data = data.data;
	};
	// Get html dataresult from ajax call to method result.
    function dataResultGetHtml(data, rfResult) {//rfResult musí byť inicializovaný
        rfResult.result = data.result;
        rfResult.errNumber = data.errNumber;
        rfResult.description = data.description;
        rfResult.data = unescape(data.data);
    };
    function z_GetAjaxException(jqXHR, exception, errorThrown) {
        var exc = OBERONApplicationBase.myException();
        if (jqXHR.status === 0) {
            exc = OBERONApplicationBase.myException('Server webovej služby je v tejto chvíli nedostupný.','',111);
        } else if (jqXHR.status == 404) {
            exc = OBERONApplicationBase.myException('Metóda pre spracovanie požiadavky nebola nájdená.', '', 404);
        } else if (jqXHR.status == 401) {
            exc = OBERONApplicationBase.myException('Užívateľ nie je prihlásený.', '', 401);
        } else if (jqXHR.status == 500) {
            exc = OBERONApplicationBase.myException('Interná chyba servera webovej služby.', '', 500);
        } else if (exception === 'parsererror') {
            exc = OBERONApplicationBase.myException('Chyba pre spracovaní JSON dát zo servera.', '', 900);
        } else if (exception === 'timeout') {
            exc = OBERONApplicationBase.myException('Čas pre spracovanie požiadavky vypršal.', '', 901);
        } else if (exception === 'abort') {
            exc = OBERONApplicationBase.myException('Asynchrónne volanie bolo prerušené.', '', 902);
        } else {
            exc = OBERONApplicationBase.myException('Neznáma chyba.<br/>' + jqXHR.statusText, '', 99);
        }
        return exc;
    };
	/* --- Extension methods --- */	
	//--- Convert date value to .NET webservice text value.
    Date.toRequest = function (dat, jsonNet) {
        var result = '';
        if (!dat || dat.length == 0) { return ''; }
        if (!(dat instanceof Date)) { return ''; }
        if (isNaN(dat.getTime())) { return ''; }
        if (jsonNet && jsonNet == true) {
            result = '/Date(' + Date.parse(dat) + '+0100)/';
        } else {// formát 'yyyy-mm-ddThh:mm:ss'
            result = dat.toISOString();
        }
        return result;
    };

	//--- Add days to date.
    Date.addDays = function (date, days) {
		var res = new Date(date);
        res.setDate(res.getDate() + days);
		return res;
    };
	
	//--- Add value to actual date.
    Date.prototype.add = function (sInterval, iNum) {
        var dTemp = this;
        if (!sInterval || iNum == 0) return dTemp;
        switch (sInterval.toLowerCase()) {
            case "ms":
                dTemp.setMilliseconds(dTemp.getMilliseconds() + iNum);
                break;
            case "s":
                dTemp.setSeconds(dTemp.getSeconds() + iNum);
                break;
            case "mi":
                dTemp.setMinutes(dTemp.getMinutes() + iNum);
                break;
            case "h":
                dTemp.setHours(dTemp.getHours() + iNum);
                break;
            case "d":
                dTemp.setDate(dTemp.getDate() + iNum);
                break;
            case "mo":
                dTemp.setMonth(dTemp.getMonth() + iNum);
                break;
            case "y":
                dTemp.setFullYear(dTemp.getFullYear() + iNum);
                break;
        }
        return dTemp;
    };
	//--- comapare two dates with seconds resolution.
    Date.areEqual = function (date1, date2) {//--- porovná či je rovnaký, bez tickov a detailov
        if (!date1 || !date2) { return false; }
        if (date1.getFullYear() != date2.getFullYear() || date1.getMonth() != date2.getMonth() || date1.getDay() != date2.getDay() || date1.getHours() != date2.getHours() || date1.getMinutes() != date2.getMinutes() || date1.getSeconds() != date2.getSeconds()) {
            return false;
        } else {
            return true;
        }
    };
	//--- Repeat given string n-times
    String.repeat = function (chr, count) {
        var str = "";
        for (var x = 0; x < count; x += 1) { str += chr }
        return str;
    };
	//--- Add left padding to text.
    String.prototype.padL = function (width, pad) {
        if (!width || width < 1) { return this; }
        if (!pad) { pad = " "; }
        var length = width - this.length;
        if (length < 1) { return this.substr(0, width); }
        return (String.repeat(pad, length) + this).substr(0, width);
    };
	//--- Add right padding to text.
    String.prototype.padR = function (width, pad) {
        if (!width || width < 1) { return this; }
        if (!pad) { pad = " "; }
        var length = width - this.length;
        if (length < 1) { this.substr(0, width); }
        return (this + String.repeat(pad, length)).substr(0, width);
    };
	//--- Format text.
    String.format = function (frmt, args) {
        for (var x = 0; x < arguments.length; x += 1) {
            var val = arguments[x + 1];
            if (!val || val == null) { val = ''; }
            frmt = frmt.replace("{" + x + "}", val);
        }
        return frmt;
    };
    window.OBERONApplicationBase = OBERONApplicationBase;
    window.OBERONApplicationBase.init();
}).call(window);