/**
 * Plugin Name: OBERON - Hotel reservation
 * Plugin URI: http://exalogic.sk
 * Description: OBERON Hotel reservation WordPress plugin to OBERON Api connector.
 * Version: 1.4.4
 * Author: Mário Moravčík
 * Release: April 2022 (initial September 2017)
 * License: GPL2
 */

(function () {
    	
	var $, eHotel;	
	var _bind = function (fn, me) { return function () { return fn.apply(me, arguments); }; };
	//root = typeof exports !== "undefined" && exports !== null ? exports : this;
	
	$ = jQuery;
	$.fn.extend({
		eHotel: function (options) {
			return $(this).each(function (input_field) {
				return new eHtl(this, options);
			});
		}
	});

	var eHtl = (function (elmn, options) {
		_self = this;
		_self.info = {
			version: '1.1',
			maxRooms: 5,
			pageCount: 3, // used page count, not actual page count
			pageIndexSendable: 2,
			pageIndexAvailability: 1,
			pageIndexStatus: 3,
			pageIndexError: 4 // error page should be last
		};
		_self.base_elm = elmn;
		_self.wizard = {
			control: null,
			btnNext: null,
			btnBack: null,
			btnFinish: null,
			errResult: null
		};
		_self.lastError = {isError: false, message: ''};
		_self.priceResultTable = null;
		_self.priceResultTableItems = null;
		_self.priceResultInfo = null;
		_self.priceResultCalc = null;
		_self.pageIndex = 0;
		_self.defaults = {
			priceCategoryGuest: '',
			roomTypeAllowChange: true,
			priceCalculateAccommodation: true,
			priceCalculateTouristTax: true,
			priceCalculateServices: true
		};
		_self.cache = { 
			roomTypes: [],
			rooms: [],
			priceCategoryGuests: []
		};
		this.init = function (options) {//init						
			_self.defaults = {
				lang: 'sk',
				key: "",
				parentIframeID: '',
				apiBase: 'oberonweb/v' + _self.info.version + '/hotel',
				wpBaseUrl: '/wp-json/', //this must be set by rest controller settings
				preferedSubmissionType: 1
			};

			if (options) {
				$.extend(_self.defaults, options);
			}

			_self.defaults.priceCategoryGuest = '';
			_self.defaults.roomTypeAllowChange = true;
			_self.defaults.priceCalculate = true;

			loadCache();
			reservation_form_init();			
			return this;
		};
		function getUrl(method) {
			var url = $('link[rel="https://api.w.org/"]').eq(0).attr('href');
			if (url == 'undefined'){ url = ''; }
			url += _self.defaults.apiBase + method;
			return url;
		};
		function reservation_form_init() {			
			// preklady
			var text = $.lingua("info_ReservationType_1");
			$('#t_submission_type').append($("<option></option>").attr("value", '1').text(text));
			text = $.lingua("info_ReservationType_2");
			$('#t_submission_type').append($("<option></option>").attr("value", '2').text(text));
			$('#t_submission_type').val(_self.defaults.preferedSubmissionType);

			$('#t_rooms').empty();
			for (var i = 1; i < 7; i += 1) {
				switch (i){
					case 1:
						text =  i + ' ' + $.lingua("info_room_1"); break;
					case 2:
					case 3:
					case 4:
						text =  i + ' ' + $.lingua("info_room_2"); break;
					case 5:
						text =  i + ' ' + $.lingua("info_room_10"); break;
					case 6:
						text = $.lingua("info_MoreRooms"); break;
					default:
						text = ''; break;
				}
				if (text != ''){							
					$('#t_rooms').append($("<option></option>").attr("value", i).text(text));
				}			
			}
			var d = new Date().setHours(0,0,0,0), d1 = Date.addDays(d, 1), d2 = Date.addDays(d, 2);
			$("#t_date_from").datepicker({ dateFormat: "dd.mm.yy", language: _self.defaults.lang, buttonImageOnly: true, minDate: new Date() }).val(OBERONApplicationBase.dateFormat(d1));
			$("#t_date_to").datepicker({ dateFormat: "dd.mm.yy", language: _self.defaults.lang, minDate: d1 }).val(OBERONApplicationBase.dateFormat(d2));

			$(".locked").prop('readonly', true);

			$("#t_rooms").on('change', function (evt) { id=evt.currentTarget; ev_room_count_click(id) });
			$("#t_submission_type").on('change', function (evt) {
				var value = $(evt.currentTarget).val();
				var text = (value === '1' ? $.lingua("info_ReservationType_1") : $.lingua("info_ReservationType_2"));
				$("#widget_header_reservation").html(text);
			});
			
			$("#btn_calculation").on("click", function (evt) { availability_info_working(); reservation_send_calculation(evt); });

			ev_room_count_click('#t_rooms'); // simulujeme klik			
			$("#room_list").show();

			_self.wizard.control = $('#reservation-slider');	
			_self.wizard.control.carousel({ interval: 0, pause: true });
			_self.wizard.control.on('slide.bs.carousel', function (evt) { evh_Wizard_SlideEvent(evt); });

			_self.wizard.btnNext = $('#btn_next');
			_self.wizard.btnBack = $('#btn_prev');
			_self.wizard.btnFinish = $('#sendRequest');
			_self.wizard.errResult = $('#oberon-reservation-error');
			
			_self.info.pageIndexError = _self.wizard.control.find('.carousel-item').length - 1; // last page is error
			
			_self.wizard.btnBack.prop('disabled', true);
			$('#btn_prev').on("click", function () { 
				ev_wizard_slide_back() 
			});			
			_self.wizard.btnNext.prop('disabled', false);
			$('#btn_next').on("click", function () { 
				ev_wizard_slide_next() 
			});
			_self.wizard.btnFinish.addClass('disabled').prop('disabled', true);
			_self.wizard.btnFinish.on("click", function (evt) { 
				reservation_send_new(evt); 
			});

			_self.priceResultTable = _self.wizard.control.find('#calculation-price-table');
			_self.priceResultTableItems = _self.wizard.control.find('#calculation-price-table-items');
			_self.priceResultInfo = _self.wizard.control.find('#calculation-result');
			_self.priceResultCalc = _self.wizard.control.find('#t_calculatedprice');
			_self.reservationResultInfo = _self.wizard.control.find('#oberon-reservation-result');
			
			if (_self.lastError.isError === true ){
				OBERONApplicationBase.wait(1000, function(){ wizard_set_error(null); });
			}
		};

		/* ------ COMMUNICATION ------- */		
		function reservation_send_new(evt) {
			//--- odoslanie 
			var obj = validation_reservation_object_load();
			var res = validate_reservation_object(obj, 2);
			if (!res.result) { return false; }
			
			ev_wizard_slide_next();

			var reservation = oberon_web_reservation_object_init(obj);
			var ajaxurl = getUrl("/SetHotel_RoomReservation");
			var data = OBERONApplicationBase.dataResult(),
				exc = OBERONApplicationBase.myException(),
				param = { 
					HotelRoomReservation: {
						PriceCalculateAccommodation: _self.defaults.priceCalculateAccommodation,
						PriceCalculateTouristTax: _self.defaults.priceCalculateTouristTax,
						PriceCalculateServices: _self.defaults.priceCalculateServices,
			        	PriceCategoryGuestName: _self.defaults.priceCategoryGuest,						
						PriceItems: [],
			        	HotelReservation: reservation
					} 
				};	
			exc = OBERONApplicationBase.postData(ajaxurl, param, data);
			reservation_result_generate(reservation, exc, data);
		};

		function reservation_send_calculation(evt) {
			//--- odoslanie 
			var obj = validation_reservation_object_load();
			var res = oberon_web_reservation_object_init(obj);
			//--- už bol validovaný
			var ajaxurl = getUrl("/GetHotel_RoomReservationAvailability");
			var data = OBERONApplicationBase.dataResult(),
				exc = OBERONApplicationBase.myException(),
				param = { 
					HotelRoomReservationAvailabilityArg: {
			        	PriceCalculate: _self.defaults.priceCalculate,
			        	PriceCategoryGuestName: _self.defaults.priceCategoryGuest,
			        	RoomTypeAllowChange: _self.defaults.roomTypeAllowChange,
			        	HotelReservation: res
					}
				};	
			exc = OBERONApplicationBase.postData(ajaxurl, param, data);
			if (!exc) {
				var text = $.lingua("err_ReservationCheck");
				exc = OBERONApplicationBase.myException(text);
			}
			if (exc.result == false) {
				var text1 = $.lingua("err_l_Number"), text2 = $.lingua("err_l_Text");
				_self.lastError.isError = true;
				_self.lastError.message = text1 + " " + exc.errNumber + "\n" + text2 + " " + exc.description;
				wizard_set_error(null);
			}	
			
			if (!data) {
				return;
			} else if (data.result) {
				availability_info_generate( data.data );
			}
		};

		function loadCache() {
			var ajaxurl = getUrl("/GetHotel_RoomTypes"),
				param = {},
			    data = OBERONApplicationBase.dataResult(),
				exc = OBERONApplicationBase.myException(),
			exc = OBERONApplicationBase.getData(ajaxurl, data);
			if (!exc) {
				var text = $.lingua("err_LoadHotelData");				
				_self.lastError.isError = true;
				_self.lastError.message = text;
			}
			if (exc.result == false) {				
				OBERONApplicationBase.logException(exc);
				_self.lastError.isError = true;
				_self.lastError.message = exc.description;
			}
			_self.cache.roomTypes = data.data; //predanie dát používateľa
			ajaxurl = getUrl("/GetHotel_PriceCategoryGuests");
			data = OBERONApplicationBase.dataResult();
			exc = OBERONApplicationBase.myException();
			param = { PriceCategoryGuestsArg: { LoadDisabled:false } };
			exc = OBERONApplicationBase.postData(ajaxurl, param, data);
			if (!exc) {
				var text = $.lingua("err_LoadHotelData");				
				_self.lastError.isError = true;
				_self.lastError.message = text;
			}
			if (exc.result == false) {				
				OBERONApplicationBase.logException(exc);
				_self.lastError.isError = true;
				_self.lastError.message = exc.description;
			}
			_self.cache.priceCategoryGuests = data.data;
			if(_self.cache.priceCategoryGuests && _self.cache.priceCategoryGuests.length!=0){
				_self.cache.priceCategoryGuests.forEach(function (item, index) {
					if (item && !item.Disabled) {
						if (item.DefaultCategory) {
							_self.defaults.priceCategoryGuest = item.Name;
						}
					}
				});
			}
		};

		function getRoomType(name) {
			var rItem = null;
			_self.cache.roomTypes.forEach(function (item, index) {
				if (item.Name == name) {
					rItem = item;
				}
			});
			return rItem;
		};

		/* ------ EVENT HANDLERS / MAPPERS ------- */
		function evh_Wizard_SlideEvent(evt) {
			if ( evt == null || evt.relatedTarget == null ) return;
			if ( evt.from == 0 && evt.relatedTarget.id == 'carousel-ohr-verify' ) { // automaticky spustiť overenie
				availability_info_working();
				reservation_send_calculation(evt);
			}
		};

		function ev_wizard_slide_next(){
			var obj = validation_reservation_object_load();
			var res = validate_reservation_object(obj, _self.pageIndex); // validujeme predošlú stranu
			if (res.result === false) { return; }
			_self.pageIndex += 1;			
			wizard_set_controls('next', obj);
			_self.wizard.control.carousel(_self.pageIndex);
		};

		function ev_wizard_slide_back(){
			var obj = validation_reservation_object_load();
			_self.pageIndex -= 1;			
			wizard_set_controls('back', obj);
			_self.wizard.control.carousel(_self.pageIndex);			
		};

		function wizard_set_controls(direction, obj){	
			var str = $(obj.Rooms_Elm).val();

			if (_self.pageIndex !== _self.info.pageIndexError){ 
				if (parseInt(str) > _self.info.maxRooms && _self.pageIndex == _self.info.pageIndexAvailability) {
					//--- preskočenie druhej stránky
					if (direction == 'next') {
						_self.pageIndex = _self.info.pageIndexSendable; 
					} else {
						_self.pageIndex = 0;
					}
				}
			}
			if (_self.pageIndex > _self.info.pageCount && _self.pageIndex !== _self.info.pageIndexError) { _self.pageIndex = _self.info.pageCount; }
			if (_self.pageIndex < 0) { _self.pageIndex = 0; }
			// set controls		
			if (_self.pageIndex === 0) {
				//page 1
				_self.wizard.btnBack.addClass('disabled').prop('disabled', true);
				availability_info_clear();
			} else if (_self.pageIndex === _self.info.pageIndexAvailability) { //dostupnosť
				_self.wizard.btnBack.removeClass('disabled').prop('disabled', false);
				_self.wizard.btnNext.removeClass('disabled').prop('disabled', false);
				_self.wizard.btnFinish.addClass('disabled').prop('disabled', true);

			} else if (_self.pageIndex === _self.info.pageIndexSendable) { //odosielanie
				_self.wizard.btnBack.removeClass('disabled').prop('disabled', false);
				_self.wizard.btnNext.addClass('disabled').prop('disabled', true);				    
				_self.wizard.btnFinish.removeClass('disabled').prop('disabled', false);

			} else if (_self.pageIndex == _self.info.pageIndexStatus || _self.pageIndex == _self.info.pageIndexError) { //výsledok odoslania alebo chyba
				_self.wizard.btnBack.addClass('disabled').prop('disabled', true);
				_self.wizard.btnNext.addClass('disabled').prop('disabled', true);				    
				_self.wizard.btnFinish.addClass('disabled').prop('disabled', true);
			}
		};

		function wizard_set_error(exc){
			var msg, ico = '<i class="icon-cancel-circled"></i>';			
			_self.wizard.errResult.children('.oberon-error-icon').html(ico);		
			if (exc == null) {
				msg = _self.lastError.message;
			} else {
				msg = exc.description;
			}
			_self.wizard.errResult.children('.oberon-error-message').html(msg);	
			_self.pageIndex = _self.info.pageIndexError;	
			wizard_set_controls( 'next', 0);
			_self.wizard.control.carousel(_self.pageIndex);
		};

		/* Udalosť vyvolaná zmenou počtu izieb rezervácie */
		function ev_room_count_click(id) {
			var value = $(id).val();
			var rmlst = $("#room_list");
			rmlst.slideUp()
			setTimeout(function () {
				rmlst.html("");
				if (parseInt(value) > _self.info.maxRooms) {
					var text = $.lingua("info_more_rooms");
					rmlst.html('<div class="roomitems"><p style="padding: 20px;">' + text + '</p></div>');
				} else {					
					template_roomtypes_init(parseInt(value));
				}
				rmlst.slideDown();
			}, 300);
		};

		/* --- Udalosť vyvolaná zmenou hodnoty v niektorom combo boxe zoznamu izieb --- */
		function ev_roomlist_item_select_click(id) {
			var eid = $(id), tx = eid.val(), index = 0, ch = 0, tpl = '';
			index = eval(eid[0].id.substring(eid[0].id.lastIndexOf('_') + 1));
			if (eid[0].id.indexOf("roomtype") > -1) {
				//vyhľadanie typu izby
				var rtype = getRoomType(tx);
				if (rtype != null) {
					$("#t_adults_" + index).val(rtype.NumberBeds);
					$("#t_children_" + index).val(rtype.NumberExtraBeds);
					//--- dogenerovať polia pre deti
					$("#c_childrenages_" + index).html("");
					if (rtype.NumberExtraBeds > 0) {
						tpl = template_roomlist_item_children_age_generate(index, rtype.NumberExtraBeds);
						$("#c_childrenages_" + index).append(tpl);
					}
				}
			} else if (eid[0].id.indexOf("adults") > -1) {
				ch = eval(tx);
			} else if (eid[0].id.indexOf("children") > -1) {
				ch = 0;
				if (tx != "-") {
					ch = eval(tx);
				}
				$("#c_childrenages_" + index).html("");
				if (ch > 0) {
					tpl = template_roomlist_item_children_age_generate(index, ch);
					$("#c_childrenages_" + index).append(tpl);
				}
			}
		};

		/* --- namapovania udalostí typov izieb --- */
		function map_events_roomlist_items(){
			$("#room_list select").on("change", function (evt) { id=evt.currentTarget;  ev_roomlist_item_select_click(id); });
		};

		/* --- odmapovanie udalostí typov izieb --- */
		function unmap_events_roomlist_items(){
			$("#room_list select").off("change");							
		};


		/* ------  ------- */
		function availability_info_clear(){
			_self.priceResultInfo.removeClass("alert-warning").removeClass("alert-success").addClass("alert-light");
			_self.priceResultInfo.empty();
			_self.priceResultInfo.hide();
			_self.priceResultTable.hide();
			_self.priceResultTableItems.empty();
			_self.priceResultCalc.val(0);
		};

		function availability_info_working(){
			availability_info_clear();
			_self.priceResultInfo.removeClass("alert-success").removeClass("alert-light").addClass("alert-warning");
			_self.priceResultInfo.html($.lingua("info_ReservationChecking"));
			_self.priceResultCalc.val(0);	
			_self.priceResultInfo.show();
		};

		function availability_info_generate(data){
			if (data.TermIsFree && !data.TermCanBeReserved) {				
				var text = $.lingua("info_ReservationCheck_3");				
				if( data.ValidationItems != null && data.ValidationItems.count != 0 ) {
					var en = [];
					data.ValidationItems.forEach(function (itm, index) {	
						if( en.indexOf( itm.Number ) > -1 ) return;
						if ( itm == null || itm.Number == 0 ) {
							text += $.lingua("info_ResValidation_XXXXXX"); 
						} else {
							var vl = itm.Value;
							text += '<br/>';
							if ( vl == null || vl.length == 0 ) {
								text +=   $.lingua("info_ResValidation_" + itm.Number);
							} else {
								text +=  String.format($.lingua("info_ResValidation_" + itm.Number), vl);
							}							
						}
						en.push(itm.Number);
					});					
				} else {
					text += "<br/>" + data.TermCanBeReservedDescription.replace(/(?:\r\n|\r|\n)/g, '<br/>');
				}
				_self.priceResultInfo.removeClass("alert-success").removeClass("alert-light").addClass("alert-warning");
				_self.priceResultInfo.html(text);
				_self.priceResultCalc.val(0);
			} else if (data.TermIsFree) {
				var str = "";
				if (data.Price_Total != 0) {
					var text = $.lingua("info_PriceCalculation");
					str = String.format("<br /><br /> " + text + " <b>{0}&nbsp;EUR</b>.", OBERONApplicationBase.numFormat(data.Price_Total, 2));
				}
				var text = $.lingua("info_ReservationCheck_1");
				_self.priceResultInfo.removeClass("alert-warning").removeClass("alert-light").addClass("alert-success");
				_self.priceResultInfo.html(String.format(text + " {0}", str));
				_self.priceResultInfo.slideDown();
				_self.priceResultCalc.val(data.Price_Total);										

				if (data.PriceItems && data.PriceItems.length != 0) {					
					_self.priceResultTable.slideDown();
					var pthtml = '';
					for (var i = 0; i < data.PriceItems.length; i++) {
						var itm = data.PriceItems[i];
						pthtml += '<tr><td>' + itm.Name + '</td><td>' + OBERONApplicationBase.numFormat(itm.Price_WithVAT_Unit, 2) + '</td><td>' + itm.Amount_Unit + '</td><td>' + itm.Unit + '</td><td>' + OBERONApplicationBase.numFormat(itm.Price_WithVAT, 2) + '</td></tr>';							
					}
					_self.priceResultTableItems.html(pthtml);
				}
			} else {
				var text = $.lingua("info_ReservationCheck_2");
				_self.priceResultInfo.removeClass("alert-success").removeClass("alert-light").addClass("alert-warning");
				_self.priceResultInfo.html(text);
				_self.priceResultCalc.val(0);
			}
			_self.priceResultInfo.show();			
		};

		function reservation_result_generate(reservation, exc, data){
			var msg = '';
			if (!exc) {
				var text = $.lingua("err_ReservationSend");
				exc = OBERONApplicationBase.myException(text);				
			} else if (exc.result == false) {				
				exc.description = $.lingua("err_l_Number") + " " + exc.errNumber + "\n" + $.lingua("err_l_Text") + " " + exc.description;
			}
			if (exc.result == false) {
				wizard_set_error(exc);
				return;
			}
			if(!data) {
				msg = $.lingua("info_ReservationError_1");	
				_self.priceResultInfo.removeClass("alert-success").removeClass("alert-light").addClass("alert-danger");
			} else if(!data.result) {
				msg = data.description;	
				_self.priceResultInfo.removeClass("alert-success").removeClass("alert-light").addClass("alert-danger");		
			} else {
				msg = (reservation.ReservationState === 1 ? $.lingua("info_ReservationOK_1") : $.lingua("info_ReservationOK_2"));
				msg = String.format(msg, reservation.Email);
				_self.priceResultInfo.removeClass("alert-danger").removeClass("alert-light").addClass("alert-success");		
			}
			_self.reservationResultInfo.html(msg);			
			_self.reservationResultInfo.slideDown();			
		};
		
		/* ------ VALIDATION - VALIDATORS ------- */						
		function validate_reservation_object(data, page) {
			//--- odoslanie 
			var today = new Date();
			today.setHours(0, 0, 0, 0);
			var rt = null, res = new OBERONApplicationBase.result();
			res.result = true; //default bez chyby
			if (!data) {
				res.description = $.lingua("err_ReservationObject");
				return res;
			}
			var obj = validation_reservation_object_init(data);
			if (page === 1) {// druhá strana nemá čo validovať
				res.result = true;
				return res;
			}
			if (page === 0) {// prvá strana
				if (isNaN(obj.DateArrival) || obj.DateArrival < today) {
					validation_control_error(obj.DateArrival_Elm);
					res.result = false;
				} else { validation_control_valid(obj.DateArrival_Elm); }
				if (isNaN(obj.DateDeparture) || obj.DateDeparture < today || obj.DateDeparture === obj.DateArrival) {
					validation_control_error(obj.DateDeparture_Elm);
					res.result = false;
				} else { validation_control_valid(obj.DateDeparture_Elm); }
				if (!obj.Rooms) {
					validation_control_error(obj.Rooms_Elm);
					res.result = false;
				} else { validation_control_valid(obj.Rooms_Elm); }
				//-- izby				
				if (obj.Rooms && obj.Rooms.length > 0) {
					for (var i = 0; i < obj.Rooms.length; i++) {
						if (!obj.Rooms[i].RoomTypeName || obj.Rooms[i].RoomTypeName.length === 0) {
							validation_control_error(obj.Rooms[i].RoomTypeName_Elm);
							res.result = false;
						} else {
							validation_control_valid(obj.Rooms[i].RoomTypeName_Elm);
							rt = getRoomType(obj.Rooms[i].RoomTypeName);
						}
						if (!obj.Rooms[i].Beds || obj.Rooms[i].Beds === 0) {
							validation_control_error(obj.Rooms[i].Adults_Elm);
							res.result = false;
						} else { validation_control_valid(obj.Rooms[i].Adults_Elm); }


						if (typeof obj.Rooms[i].ExtraBeds == 'undefined' && (!rt || rt.NumberExtraBeds > 0)) {
							validation_control_error(obj.Rooms[i].Children_Elm);
							validation_control_error("#c_childrenages_" + (i + 1));
							res.result = false;
						} else { validation_control_valid(obj.Rooms[i].Children_Elm); }
						if (obj.Rooms[i].ExtraBeds && obj.Rooms[i].ExtraBeds > 0) { //validujeme veky
							if (!obj.Rooms[i].ChildrenAge || obj.Rooms[i].ChildrenAge.length < obj.Rooms[i].ExtraBeds) {
								validation_control_error("#c_childrenages_" + (i + 1) + " .children-" + (i + 1));
								res.result = false;
							} else {
								validation_control_error_clear("#c_childrenages_" + (i + 1) + " .children-" + (i + 1));
								for (var j = 0; j < obj.Rooms[i].ChildrenAge.length; j++) {
									if (obj.Rooms[i].ChildrenAge[j] < 1) {
										validation_control_error("#t_children_" + (j + 1) + '' + (i + 1) );										
										res.result = false;
									} else { validation_control_valid("#t_children_" + (j + 1) + '' + (i + 1)); }
								}
								if (res.result === true) { validation_control_valid("#c_childrenages_" + (i + 1) + " .children-" + (i + 1)); }
							}
						}
					}
				}
			}
			if (page === 2) { // overenie pred odoslaním
				if (!obj.Name || obj.Name.length < 2) {
					validation_control_error(obj.Name_Elm);
					res.result = false;
				} else { validation_control_valid(obj.Name_Elm); }
				if (!obj.FamilyName || obj.FamilyName.length < 2) {
					validation_control_error(obj.FamilyName_Elm);
					res.result = false;
				} else { validation_control_valid(obj.FamilyName_Elm); }
				if (!obj.Email || !OBERONApplicationBase.validateEmail(obj.Email)) {
					validation_control_error(obj.Email_Elm);
					res.result = false;
				} else { validation_control_valid(obj.Email_Elm); }
			}
			return res;
		};

		function validation_control_error(element) {
			var elm = null;
			if (element.substring(0,1)=='#'){
				elm = $(element);
			} else {
				elm = $(element).closest('.form-control');
			}
			elm.removeClass('is-valid').addClass('is-invalid');
		};

		function validation_control_error_clear(element) {
			var elm = null;
			if (element.substring(0,1)=='#'){
				elm = $(element);
			} else {
				elm = $(element).closest('.form-control');
			}
			elm.removeClass('is-invalid');
		};
		
		function validation_control_valid(element) {
			var elm = null;
			if (element.substring(0,1)=='#'){
				elm = $(element);
			} else {
				elm = $(element).closest('.form-control');
			}
			elm.removeClass('is-invalid').addClass('is-valid');
		};

		/* ------ HTML GENERATION ------- */
		/* --- Inicializácia typov izieb podľa počtu --- */
		function template_roomtypes_init(count) {
			unmap_events_roomlist_items(); // odmapovať
			for (var i = 1; i < count + 1; i += 1) {
				$("#room_list").append(template_roomlist_item_generate(i));
			}			
			map_events_roomlist_items(); // namapovať
		};

		function template_roomlist_item_generate(id) {
			//typ izby
			var text = $.lingua("info_room_type");
			var i = 0;

			var tpl = '<!-- room item row start --><div class="form-row">';
			tpl += '<div id="roomitem_' + id + '" class="form-group col-md">';
			if (id < 2) {
				tpl += '<label class="control-label" for="t_roomtype_' + id + '">' + text + '</label>';
			}			
			tpl += '<div class="input-group mb-2 mb-sm-0">';
			tpl += '<select  class="form-control" id="t_roomtype_' + id + '" name="t_roomtype_' + id + '"><option></option>';
			_self.cache.roomTypes.forEach(function (item, index) {
				tpl += '<option>' + item.Name + '</option>';
			});
			tpl += '</select></div></div>';
			// dospelí	
			tpl += '<div class="form-group col-md-1">';
			if (id < 2) {
				text = $.lingua("info_Adults");
				tpl += '<label class="control-label" for="t_adults_' + id + '">' + text + '</label>';
			}
			tpl += '<div class="input-group mb-2 mb-sm-0">';
			tpl += '<select  class="form-control" id="t_adults_' + id + '" name="t_adults_' + id + '"><option></option>';
			for (i = 1; i < 6; i++) {
				tpl += '<option>' + i + '</option>';
			}
			tpl += '</select></div></div>';
			// deti počet
			tpl += '<div class="form-group col-md-1">';
			if (id < 2) {
				text = $.lingua("info_Children");
				tpl += '<label class="control-label" for="t_children_' + id + '">' + text + '</label>';
			}
			tpl += '<div class="input-group mb-2 mb-sm-0">';
			tpl += '<select  class="form-control" id="t_children_' + id + '" name="t_children_' + id + '"><option></option>';
			for (i = 0; i < 6; i++) {
				tpl += '<option>' + i + '</option>';
			}					
			tpl += '</select></div></div>';
			// deti veky
			tpl += '<div class="form-group col-md">';
			if (id < 2) {
				text = $.lingua("info_ChildrenAges");
				tpl += '<label class="control-label" for="t_children_1' + id + '">' + text + '</label>';
			}					
			tpl += '<div class="input-group mb-2 mb-sm-0" id="c_childrenages_' + id + '">';
			tpl += '<input type="text" class="form-control" id="t_children_1' + id + '" name="t_children_1' + id + ' />\n';
			tpl += '</div></div></div>';
			tpl += '<!-- room item row end -->';

			return tpl;
		};

		function template_roomlist_item_children_age_generate(id, count) {
			var tpl = '';
			for (var i = 1; i < count + 1; i += 1) {
				tpl += '<input type="text" class="form-control children-' + id + '" id="t_children_'+ i + '' + id + '" name="t_children_' + i + '' + id + '" />';
			}
			return tpl;
		};

		/* ------ VALIDATION - OBJECTS ------- */
		/* --- Načítanie údajov formulára do objektu pre overenie údajov --- */
		function validation_reservation_object_load() {
			var dat = new Date(), str = '',
				res = new validation_reservation_object_init();

			res.ReservationState = parseInt($(res.ReservationState_Elm).val());
			//res.DateArrival = '/Date(' + Date.parse(OBERONApplicationBase.dateGet($(res.DateArrival_Elm).val())) + '+0100)/';
			//res.DateDeparture = '/Date(' + Date.parse(OBERONApplicationBase.dateGet($(res.DateDeparture_Elm).val())) + '+0100)/';
			res.DateArrival = OBERONApplicationBase.dateGet($(res.DateArrival_Elm).val());
			res.DateDeparture = OBERONApplicationBase.dateGet($(res.DateDeparture_Elm).val());
			res.Address_City = $(res.Address_City_Elm).val();
			res.Address_Street = $(res.Address_Street_Elm).val();
			res.Address_PostalCode = $(res.Address_PostalCode_Elm).val();
			res.Address_Country = $(res.Address_Country_Elm).val();
			res.Name = $(res.Name_Elm).val();
			res.FamilyName = $(res.FamilyName_Elm).val();
			res.Email = $(res.Email_Elm).val();
			res.Phone = $(res.Phone_Elm).val();
			res.Notice = $(res.Notice_Elm).val();
			res.ReservationSourceName = "Priamy kontakt";			    
			res.Rooms = [];
			res.Calculated_Price = eval($(res.Calculated_Price_Elm).val());
			str = $(res.Rooms_Elm).val();
			if (parseInt(str) < 10) {
				var cnt = eval(str.substring(0, str.indexOf(' ')));
				if (!cnt) { cnt = eval(str); }
				for (var i = 1; i != cnt + 1; i += 1) {
					var ri = validation_reservation_room_object_init(i);

					ri.RoomTypeName = $(ri.RoomTypeName_Elm).val();
					ri.Beds = eval($(ri.Adults_Elm).val());
					ri.ExtraBeds = eval($(ri.Children_Elm).val());
					ri.ChildrenAge = [];
					if (ri.ExtraBeds > 0) {
						$(".children-" + i).each(function (index, item) {
							var num = eval($(item).val());
							if (typeof num != 'undefined') {
								ri.ChildrenAge.push(num);
							} else { ri.ChildrenAge.push(0); }
						});
					}
					res.Rooms.push(ri);
				}
			}
			return res;
		};

		function validation_reservation_object_init(data) {
			var obj = null;
			if (data) {
				obj = data;
			} else {
				obj = {
					ReservationState_Elm: '#t_submission_type',
					DateArrival_Elm: '#t_date_from',
					DateDeparture_Elm: '#t_date_to',
					Calculated_Price_Elm: '#t_calculatedprice',
					Name_Elm: '#t_name',
					FamilyName_Elm: '#t_namefamily',
					Email_Elm: '#t_email',
					Notice_Elm: '#t_notice',
					Phone_Elm: '#t_phone',
					Address_Street_Elm: '#t_street',
					Address_City_Elm: '#t_city',
					Address_Country_Elm: '#t_country',
					Address_PostalCode_Elm: '#t_postalcode',
					Rooms_Elm: "#t_rooms",
					/* ReservationState - typ rezervácie
					0 - nie je zadaný stav (zmluvné rezervácie)		
					1 - Dopyt
					2 - Nová On-line rezervácia  */
					ReservationState: 0,
					DateArrival: '',
					DateDeparture: '',
					Rooms: [], //indexovaná
					Calculated_Price: 0,
					PartnerNumber: 0,
					Name: '',
					FamilyName: '',
					Email: '',
					Notice: '',
					Phone: '',
					Address_Street: '',
					Address_City: '',
					Address_Country: '',
					Address_PostalCode: '',
					ReservationSourceName: ''
				};
			}
			return obj;
		};

		function validation_reservation_room_object_init(index) {
			var obj = {
				RoomTypeName_Elm: '#t_roomtype_' + index,
				Adults_Elm: '#t_adults_' + index,
				Children_Elm: '#t_children_' + index,
				ChildrenAge_Elm: '#t_children_' + index, //indexovaná
				PreferredRoomName_Elm: '' + index,

				RoomTypeName: '',
				Beds: 0,
				ExtraBeds: 0,
				ChildrenAge: [0],
				PreferredRoomName: ''
			};
			return obj;
		};

		/* ------ OBERON WEB SERVICE OBJECTS ------- */
		/* --- Objekt rezervácie odosielaný do OBERON Web a mapovanie hodnôt. --- */
		function oberon_web_reservation_object_init(validationObject) {
			if (!validationObject) { return null; }
			var res = {
				ReservationState: validationObject.ReservationState,
				DateArrival: Date.toRequest(validationObject.DateArrival, false),
				DateDeparture: Date.toRequest(validationObject.DateDeparture, false),
				Address_City: validationObject.Address_City,
				Address_Street: validationObject.Address_Street,
				Address_PostalCode: validationObject.Address_PostalCode,
				Address_Country: validationObject.Address_Country,
				Name: validationObject.Name,
				FamilyName: validationObject.FamilyName,
				Email: validationObject.Email,
				Phone: validationObject.Phone,
				Notice: validationObject.Notice,
				ReservationSourceName: validationObject.ReservationSourceName,
				Rooms: []
			};
			if (validationObject.Rooms) { // naplnenie izieb
				for (var i = 0; i < validationObject.Rooms.length; i += 1) {
					res.Rooms.push({
						RoomTypeName: validationObject.Rooms[i].RoomTypeName,
						Beds: validationObject.Rooms[i].Beds,
						ExtraBeds: validationObject.Rooms[i].ExtraBeds == null ? 0 : validationObject.Rooms[i].ExtraBeds,
						ChildrenAge: validationObject.Rooms[i].ChildrenAge,
						PreferredRoomName: validationObject.Rooms[i].PreferredRoomName
					});
				}
			}
			return res;
		};

		this.init(options);
		return eHtl;
	});
}).call(this);