/**
 * Plugin Name: OBERON - Hotel reservation plugin settings page
 * Plugin URI: http://exalogic.sk
 * Description: OBERON Hotel reservation WordPress plugin to OBERON Api connector.
 * Version: 1.4.4
 * Author: Mário Moravčík
 * Release: July 2021 (September 2020)
 * License: GPL2
 */
(function () {    	
	var $;	
	var _bind = function (fn, me) { return function () { return fn.apply(me, arguments); }; };		
	$ = jQuery;
	$.fn.extend({
		rpAdminPage: function (options) {
			return $(this).each(function (input_field) {
				return new eModule(this, options);
			});
		}
	});
	var eModule = (function (elmn, options) {
		_self = this;
		_self.info = {
			apiVersion: '1.1', // This has to be same as rest controller version
			date: '3.8.2018'			
		};
		_self.base_elm = $(elmn);
		_self.settings = {};
		_self.defaults = {};
		_self.cache = { 
			roomTypes: [],
			rooms: [],
			priceCategoryGuests: []
		};
		_self.controls = {
			messageContainer: null,
			calculateAccommodation: null,
			priceCategoryGuests: null,
			testConnection: null
		};
		this.init = function (options) {//init						
			_self.defaults = {
				lang: "sk",
				key: "",
				parentIframeID: '',
				nonce: '',
				root: '',
				apiBase: 'oberonweb/v' + _self.info.apiVersion + '/hotel',
				wpBaseUrl: '/wp-json/', //this must be set by rest controller settings
				preferedSubmissionType: 1
			};
			if (options) {
				$.extend(_self.defaults, options);
			}			
			OBERONApplicationBase.settings.apiNonce = _self.defaults.nonce;
			getSettings();
			loadCache();			
			loadFormControls();
			return this;
		};
		/* ------ FORM ------- */
		function loadFormControls(){
			if ( !_self.settings ) {
				return;
			}
			_self.controls.calculateAccommodation = $(_self.base_elm.find('#' + _self.settings.calculate_accommodation.control_id));
			_self.controls.priceCategoryGuests = $(_self.base_elm.find('#' + _self.settings.price_category_guest.control_id));
			_self.controls.calculateAccommodation.on('change', function(evt){
				var val = _self.controls.calculateAccommodation.is(":checked");
				if ( val == true) {
					_self.controls.priceCategoryGuests.attr('disabled', false);
				} else {
					_self.controls.priceCategoryGuests.attr('disabled', true);
				}
			});
			_self.controls.priceCategoryGuests.empty();
			var opt =  $('<option />');
			_self.controls.priceCategoryGuests.append(opt);

			if ( _self.cache.priceCategoryGuests && _self.cache.priceCategoryGuests.length != 0 && _self.controls.priceCategoryGuests ){

				_self.cache.priceCategoryGuests.forEach(function (item, index) {
					var opt =  $('<option />').val(item.Name).text(item.Name);
					if (item.Name == _self.settings.price_category_guest.value)  { opt.attr('selected','selected'); }
					_self.controls.priceCategoryGuests.append(opt);
				});
			} else {
				_self.controls.priceCategoryGuests.val(_self.settings.price_category_guest.value);
			}

			if ( _self.settings.calculate_accommodation.value != 1) {
				_self.controls.priceCategoryGuests.attr('disabled', true);	
			}			
			_self.controls.messageContainer = $(_self.base_elm.find('.orpa-settings-message'));
			_self.controls.messageContainer.hide();
			// Tlačítko pre testovanie
			_self.controls.testConnection = $(_self.base_elm.find('#oberon-test-connection'));
			_self.controls.testConnection.on('click', function (evt) { evh_TestConnection(evt); });
		};

		/* --- Event Handlers --- */
		function evh_TestConnection(event) {
			event.preventDefault();
			if (jQuery(event.target).hasClass( 'disabled' )) { return; }
			var exc  = testServiceAvailability();
			if ( !exc.result ) {
				showMessage('e', exc.description);
				return;
			}
			showMessage('i', 'Overenie prebehlo v poriadku. Návratová hodnota z OBERON Web API: ' + exc.data);				
		}

		/* ------ SYSTEM ------- */
		function getUrl(method) {
			var url = _self.defaults.root;
			if (url == 'undefined'){ url = ''; }
			url += _self.defaults.apiBase + method;
			return url;
		};
		function loadCache() {
			var apiURL = getSettingValue('');
			//if ( !)

			var ajaxurl = getUrl("/GetHotel_RoomTypes"),
				param = {},
			    data = OBERONApplicationBase.dataResult(),
				exc = OBERONApplicationBase.myException(),
			exc = OBERONApplicationBase.getData(ajaxurl, data);
			if (!exc) {
				var text = $.lingua("err_LoadHotelData");				
				
			}
			if (exc.result == false) {				
				OBERONApplicationBase.logException(exc);
				
			}
			_self.cache.roomTypes = data.data; //predanie dát používateľa
			//
			ajaxurl = getUrl("/GetHotel_PriceCategoryGuests");
			data = OBERONApplicationBase.dataResult();
			exc = OBERONApplicationBase.myException();
			param = { PriceCategoryGuestsArg: { LoadDisabled:false } };
			exc = OBERONApplicationBase.postData(ajaxurl, param, data);
			if (!exc) {
				var text = $.lingua("err_LoadHotelData");				
				
			}
			if (exc.result == false) {				
				OBERONApplicationBase.logException(exc);				
			}
			_self.cache.priceCategoryGuests = data.data;			
		};

		function getRoomType(name) {
			var rItem = null;
			_self.cache.roomTypes.forEach(function (item, index) {
				if (item.Name == name) {
					rItem = item;
				}
			});
			return rItem;
		};
		
		function testServiceAvailability() {
			var ajaxurl = getUrl("/ping");
			var result = false;
			jQuery.ajax({ type: "GET", url: ajaxurl, cache: false, async: false, dataType: 'json', contentType: 'application/json; charset=utf-8', timeout: 1000, processData: false,
				success: function (data, textStatus, xhr) {
					if(!data || (!data.result && !data.description)) {
						result = { result: false, data: '', description: 'Nebola načítaná adresa IP. Skontrolujte nastavenie OBERON Center prípadne, či je služba dostupná.' };	
					} else if(!data.result) { 
						result = { result: false, data: '', description: data.description };	
					} else if(!OBERONApplicationBase.validateIP(data.data)) {
						result = { result: false, data: '', description: 'Nebola načítaná adresa IP. Skontrolujte nastavenie OBERON Center prípadne, či je služba dostupná.' };	
					} else {
						result = data;
					}
				},
				error: function (xhr, textStatus, errorThrown) { 
					result = { result: false, data: null, description: '' };
					var js = JSON.parse(xhr.responseText);					
					if(js) {
						result.description = js.message;
					} else {
						result.description = xhr.responseText;
					}					
				}
			});
			return result;
		};

		// Načítanie nastavení wordpress pluginu
		function getSettings() {
			var ajaxurl = getUrl("/getPluginSettings"),
			param = {},
			data = OBERONApplicationBase.dataResult(),
			exc = OBERONApplicationBase.myException(),
			exc = OBERONApplicationBase.getData(ajaxurl, data);
			if (!exc) {
				var text = $.lingua("err_LoadSettings");
			}	
			_self.settings = data.data;
		};

		function getSettingValue(settingName) {
			if ( !_self.settings || _self.settings.length === 0 ) {
				return null;
			}
			return _self.settings[settingName];
		};

		function showMessage(type, message) {
			_self.controls.messageContainer.show();			
			_self.controls.messageContainer.removeClass('alert');
			_self.controls.messageContainer.removeClass('info');

			var ico = jQuery(_self.controls.messageContainer.find('.inner-icon'));			
			if(!type || type=='i') {
				_self.controls.messageContainer.addClass('info');
				ico.html(jQuery('<span class="dashicons dashicons-info-outline info"></span>')); // 'dashicons-yes-alt'
			} else if(type=='e') {
				_self.controls.messageContainer.addClass('alert');
				ico.html(jQuery('<span class="dashicons dashicons-warning alert"></span>')); // 'dashicons-dismiss'
			}

			_self.controls.messageContainer.find('.inner-message').html(message);

			OBERONApplicationBase.wait(10000, function(){ _self.controls.messageContainer.hide(); });
		};

		this.init(options);
		return _self;
	});
}).call(this);